#include "archive_filter.h"

#include <qdir.h>

#include <kar.h>
#include <kzip.h>
#include <ktar.h>
#include <kstaticdeleter.h>
#include <ktempdir.h>
#include <klocale.h>

#include "compress_filter.h"

using namespace Konvert;

//-----------------------------------------------------------------------------
const ArchiveKDETransform::Data ArchiveKDETransform::DATA[Nb_Types] = {
  { "application/x-archive", { true, false, true }}, // @fixme checking by extracting
  { "application/x-zip",     { true, true,  true }}, // @fixme checking by extracting
  { "application/x-tar",     { true, true,  true }}  // @fixme checking by extracting
};

KArchive *ArchiveKDETransform::factory(const MimeType &mimetype,
                                       const QString &filename)
{
  QIODevice *iod = 0;
  if ( mimetype.isType(Compressed) ) {
    const CompressFilter *cf =
      static_cast<const CompressFilter *>(Filter::find("Konvert::CompressFilter"));
    Q_ASSERT(cf);
    iod = cf->factory(mimetype, filename);
  } else iod = new QFile(filename);
  switch ( type(mimetype) ) {
  case Ar: return new KAr(iod);
  case Zip: return new KZip(iod);
  case Tar: return new KTar(iod);
  case Nb_Types: break;
  }
  Q_ASSERT(false);
  return 0;
}

bool ArchiveKDETransform::doAction(TransAction action, const MimeType &mimetype,
                          const ExchangeData &source, const ExchangeData &dest,
                          Filter *filter) const
{
  qDebug("archive doAction %i %s %s", action, source.dataFilename.latin1(),
         dest.dataFilename.latin1());
  QString filename =
    (action==TransCreate ? dest.dataFilename : source.dataFilename);
  KArchive *archive = factory(mimetype, filename);
  IOType ioType = TRANS_ACTION_DATA[action].ioType;
  bool ok = false;
  if ( archive==0 || !archive->open(IOTYPE_DATA[ioType].openMode) )
    filter->done(IOTYPE_DATA[ioType].openError);
  else {
    switch (action) {
    case TransCheck:
    case TransExtract:
      archive->directory()->copyTo(dest.dataFilename);
      ok = ( archive->device()->status()==IO_Ok );
      if ( !ok ) filter->done(FilterErr, i18n("IO error."));
      break;
    case TransCreate:
      ok = ( archive->addLocalDirectory(source.dataFilename, QString::null)
             && archive->device()->status()==IO_Ok );
      if ( !ok ) filter->done(FilterErr, i18n("Error adding directory to archive."));
      break;
    case Nb_TransActions:
      Q_ASSERT(false);
      break;
    }
  }
  delete archive;
  return ok;
}

ArchiveKDETransform::Type ArchiveKDETransform::type(const MimeType &mimetype)
{
  QString amt = mimetype.data().baseMimetype;
  for (uint i=0; i<Nb_Types; i++)
    if ( DATA[i].mimetype==amt ) return Type(i);
  return Nb_Types;
}

bool ArchiveKDETransform::supports(TransAction action, const MimeType &mimetype) const
{
  if ( mimetype.isComposite() ) {
    const CompressFilter *cf =
      static_cast<const CompressFilter *>(Filter::find("Konvert::CompressFilter"));
    Q_ASSERT(cf);
    Q_ASSERT(mimetype.data().parentMimetype);
    MimeType parent(mimetype.data().parentMimetype);
    if ( !cf->isSupported(action, parent) ) return false;
  }
  Type t = type(mimetype);
  if ( t==Nb_Types ) return false;
  return DATA[t].supported[action];
}

//-----------------------------------------------------------------------------
const ProcessTransform::Data ArchiveProcessTransform::DATA[Nb_Types] = {
  {"application/x-arj", {"arj", NoOutput, 0}, {
    {true, {NoDest,     "x", "-y", 0}}, // extract + assume yes
    {true, {DestSource, "a", "-r", 0}}, // add files + recurse
    {true, {NoDest,     "t",       0}}  // test integrity
  }},
  {"application/x-arj", {"unarj", NoOutput, 0}, {
    {true,  {NoDest, "x", 0}}, // extract
    NOT_SUPPORTED,
    {true,  {NoDest, "t", 0}}  // test intergrity
  }},
  {"application/x-lha", {"lha", NoOutput, "--version", 0}, {
    {true,  {NoDest, "x", 0}}, // extract
    {false, {NoDest, "a", 0}}, // add files // @fixme do not recurse
    {true,  {NoDest, "t", 0}}  // check CRC
  }},
  {"application/x-rar", {"rar", NoOutput, 0}, {
    {true, {NoDest,     "x",              0}}, // extract
    {true, {DestSource, "a", "-r", "-rr", 0}}, // add files + recurse + recovery record
    {true, {NoDest,     "t",              0}}  // test integrity
  }},
  {"application/x-rar", {"unrar", NoOutput, 0}, {
    {true,  {NoDest, "x", 0}}, // extract
    NOT_SUPPORTED,
    {true,  {NoDest, "t", 0}}  // test integrity
  }},
  {"application/x-zoo", {"zoo", NoOutput, 0}, {
    {true,  {NoDest, "x:",      0}}, // extract (absolute paths -> relative)
    {false, {NoDest, "a",       0}}, // add files // @fixme do not recurse
    {true,  {NoDest, "-test",   0}}  // test intergrity
  }},
  {"application/x-cpio", {"afio", NoOutput, "--version", 0}, {
    {true,  {NoDest, "-i", 0}}, // extract
    // @fixme: needs a file containing a list of files... (or 'find' ...)
    {false, {NoDest, "-o", 0}}, // create
    {false, {NoDest, "-t", 0}}  // list files
  }},
  {"application/x-cpio", {"cpio", NoOutput, "--version", 0}, {
    // extract + make dir + no absolute filenames + use file instead of stdin
    {true,  {NoDest, "-i", "-d", "--no-absolute-filenames", "-F", 0}},
    // add files + use file instead of stdin
    // @fixme: needs a file containing a list of files...
    {false, {NoDest, "-o", "-F",                                  0}},
    {false, {NoDest, "-i", "--list", "-F",                        0}} // list files
  }},
  {"application/x-shar", {"unshar", NoOutput, "--version", 0}, {
    {true,  {NoDest, 0}},
    NOT_SUPPORTED,
    NOT_SUPPORTED
  }},
  {"application/x-shar", {"shar", NoOutput, "--version", 0}, {
    NOT_SUPPORTED,
    {false,  {DestSource, "-o", 0}}, // use file instead of stdout // @fixme add .01 to output...
    NOT_SUPPORTED
  }},
  {"application/x-jar", {"jar", NoOutput, 0}, {
    {true,  {NoDest,     "xf",  0}}, // extract + use filename
    {true,  {DestSource, "cfM", 0}}, // create + use filename + do not create manifest
    {false, {NoDest,     "tf",  0}}  // list files + use filename
  }},
  // @fixme "unace" seems to block and go wild when started with KProcess...
/*  {"application/x-ace", "unace", "-h", {
    {false,  "x", "-y", 0}, // extract + assume yes
    {false,            0},
    {false,  "t",       0}  // test integrity
  }},*/
  {"application/x-arc", {"arc", NoOutput, 0}, {
    {true, {NoDest,     "x", 0}}, // extract
    {true, {DestSource, "a", 0}}, // create
    {true, {NoDest,     "t", 0}}  // test integrity
  }},
  {"application/cab", {"cabextract", NoOutput, 0}, {
    {true, {NoDest,       0}}, // extract
    NOT_SUPPORTED,
    {true, {NoDest, "-l", 0}}  // list files
  }},
  {"application/x-iso", {"mkisofs", NoOutput, "-version", 0}, {
    NOT_SUPPORTED,
    {true,  {DestSource, "-o", 0}}, // create ISO-9660
    NOT_SUPPORTED
  }}
};

bool ArchiveProcessTransform::supports(TransAction action,
                                       const MimeType &mimetype) const

{
  if ( _data.mimetype!=mimetype.name() ) return false;
  return ProcessTransform::supports(action, mimetype);
}

bool ArchiveProcessTransform::doAction(TransAction action,  const MimeType &,
                                   const QString &source, const QString &dest,
                                   Filter *filter) const
{
  QString dirname = (action==TransCreate ? source : dest);
  QString dirpath = dirname;
  if ( action==TransCreate ) {
    QDir d(source);
    dirname = d.dirName();
    d.cdUp();
    dirpath = d.absPath();
  }
  QStringList a = args(action, source, dirname);
  int res = Process::run(_data.data.command, a, dirpath);
  if ( res!=0 ) filter->done(FilterErr, "Error proccessing archive.");
  return ( res==0 );
}

//-----------------------------------------------------------------------------
KStaticDeleter<TransformList> asd;
TransformList *ArchiveFilter::_list = 0;

const bool ArchiveFilter::SUPPORTED[Nb_Actions] = {
  true, // Convert
  true, // Extract
  true, // Create
  true  // Check
};

ArchiveFilter::ArchiveFilter(Action action)
  : TransformFilter(asd, _list, DirectoryURL, Archive, action)
{}

void ArchiveFilter::fillList(TransformList &list) const
{
  list.append(new ArchiveKDETransform);
  for (uint i=0; i<ArchiveProcessTransform::Nb_Types; i++)
    list.append(new ArchiveProcessTransform(i));
}

bool ArchiveFilter::isSupported(Action, TransAction tAction,
                                IOType type, const MimeType &mimetype) const
{
  if ( tAction==Nb_TransActions ) {
    if ( type==Source  && mimetype.isDirectory() ) return true;
    return false;
  }
  return TransformFilter::isSupported(tAction, mimetype);
}

void ArchiveFilter::addFilters(QPtrList<Filter> &list)
{
  for (uint i=0; i<Nb_Actions; i++)
    if ( SUPPORTED[i] )
      list.append(new ArchiveFilter(Action(i)));
}

#include "archive_filter.moc"
