#include "compress_filter.h"

#include <qfile.h>

#include <kfilterbase.h>
#include <kfilterdev.h>
#include <klocale.h>
#include <kstaticdeleter.h>

using namespace Konvert;

//-----------------------------------------------------------------------------
static KStaticDeleter<QMap<QString, bool> > supsd;
QMap<QString, bool> *CompressKDETransform::_supportedCompressions = 0;
char CompressKDETransform::BUFFER[Buffer_Size];

QIODevice *CompressKDETransform::factory(const MimeType &mimetype,
                                         const QString &source) const
{
  const char *cmt = mimetype.data().parentMimetype;
  qDebug("KDEcompress::factory %s %s", source.latin1(), cmt);
  return KFilterDev::deviceForFile(source, cmt, true);
}

bool CompressKDETransform::copy(QIODevice &source, QIODevice &dest,
                                Filter *filter)
{
  // open source
  if ( !source.open(IO_ReadOnly) ) {
    filter->done(ReadingErr);
    return false;
  }

  // open dest
  if ( !dest.open(IO_WriteOnly) ) {
    filter->done(WritingErr);
    return false;
  }

  // copy
  while ( !source.atEnd() ) {
    Q_LONG size = source.readBlock(BUFFER, Buffer_Size);
    if ( size==-1 ) {
      filter->done(ReadingErr);
      return false;
    }
    Q_LONG wsize = dest.writeBlock(BUFFER, size);
    if ( wsize==-1 || wsize!=size ) { // #### wsize!=size ???
      filter->done(WritingErr);
      return false;
    }
  }

  return true;
}

bool CompressKDETransform::doAction(TransAction action, const MimeType &mimetype,
                          const ExchangeData &source, const ExchangeData &dest,
                          Filter *filter) const
{
  qDebug("KDEcompress::doAction %i %s", action, mimetype.name().latin1());
  QIODevice *from = 0, *to = 0;
  bool ok = false;
  switch (action) {
  case TransExtract:
    from = factory(mimetype, source.dataFilename);
    to = new QFile(dest.dataFilename);
    break;
  case TransCreate:
    from = new QFile(source.dataFilename);
    to = factory(mimetype, dest.dataFilename);
    break;
  case TransCheck:
  case Nb_TransActions:
    Q_ASSERT(false);
    break;
  }
  IOType ioType = TRANS_ACTION_DATA[action].ioType;
  if ( from==0 || to==0 ) filter->done(IOTYPE_DATA[ioType].openError);
  else ok = copy(*from, *to, filter);
  delete from;
  delete to;
  return ok;
}

bool CompressKDETransform::supports(TransAction action,
                                    const MimeType &mimetype) const
{
  qDebug("KDECompress::supports %s", mimetype.name().latin1());
  if ( action==TransCheck ) return false; // no native integrity check
  if ( _supportedCompressions==0 )
    supsd.setObject(_supportedCompressions, new QMap<QString, bool>);
  const char *cmt = mimetype.data().parentMimetype;
  QMap<QString, bool>::const_iterator it = _supportedCompressions->find(cmt);
  qDebug("%s %i", cmt, (it!=_supportedCompressions->end() ? it.data() : -1));
  if ( it!=_supportedCompressions->end() ) return it.data();
  KFilterBase *fb = KFilterBase::findFilterByMimeType(cmt);
  qDebug("%i", fb);
  _supportedCompressions->insert(cmt, fb);
  delete fb;
  return fb;
}

//-----------------------------------------------------------------------------
const ProcessTransform::Data CompressProcessTransform::DATA[Nb_Types] = {
  {"application/x-lzop", {"lzop", NoOutput, "-V", 0}, {
    {true, {NoDest,     "-x",       0}}, // decompress
    {true, {DestSource, "-9", "-o", 0}}, // compress (best compression)
    {true, {NoDest,     "-t",       0}}  // test integrity
  }},
  // since gzip is natively supported: only integrity check is used
  {"application/x-gzip", {"gzip", NoOutput, "-V", 0}, {
    {true,  {NoDest, "-d",       0}}, // decompress
    {false, {NoDest, "-9", "-c", 0}}, // compress (best compression) // @fixme needs "> dest"
    {true,  {NoDest, "-t",       0}}  // test integrity
  }},
  // since bzip2 is natively supported: only integrity check is used
  {"application/x-bzip2", {"bzip2", NoOutput, "-V", 0}, {
    {true,  {NoDest, "-d",       0}}, // decompress
    {false, {NoDest, "-9", "-c", 0}}, // compress (best compression) // @fixme needs "> dest"
    {true,  {NoDest, "-t",       0}}  // test integrity
  }},
  // since compress is already supported by gzip for reading : only create is used
  {"application/x-compress", {"compress", NoOutput, "-V", 0}, {
    NOT_SUPPORTED,
    {false, {NoDest, 0}}, // compress //@fixme file is *replaced* by file.Z ... (should use stdout)
    NOT_SUPPORTED
  }}
};

bool CompressProcessTransform::supports(TransAction action,
                                        const MimeType &mimetype) const
{
  if ( _data.mimetype!=QString(mimetype.data().parentMimetype) )
    return false;
  return ProcessTransform::supports(action, mimetype);
}

QIODevice *CompressProcessTransform::factory(const MimeType &mimetype,
                                             const QString &source) const
{
  qDebug("transcompress::factory");
  ExchangeData src, dst;
  src.dataFilename = source;
  TempFile *tmp = new TempFile(TempFile::DoNotCreate, TempFile::AutoDelete);
  dst.dataFilename = tmp->name();
  if ( !doAction(TransExtract, mimetype, src, dst, 0) ) {
    delete tmp;
    return 0;
  }
  return tmp;
}

//-----------------------------------------------------------------------------
KStaticDeleter<TransformList> csd;
TransformList *CompressFilter::_list = 0;

const bool CompressFilter::SUPPORTED[Nb_Actions] = {
  true, // convert
  true, // extract
  true, // create
  true  // check
};

CompressFilter::CompressFilter(Action action)
  : TransformFilter(csd, _list, FileURL, Compressed, action)
{}

bool CompressFilter::isSupported(Action action, TransAction tAction,
                                 IOType type, const MimeType &mimetype) const
{
  if ( tAction!=Nb_TransActions ) {
    switch (action) {
    case Check:
    case Extract:
      return isSupported(tAction, mimetype);
    case Create:
      return ( !mimetype.isComposite() && isSupported(tAction, mimetype) );
    case Convert:
      return ( (type==Source || !mimetype.isComposite())
               && isSupported(tAction, mimetype) );
    default:
      Q_ASSERT(false);
      return false;
    }
  }
  return ( action==Create && type==Source && !mimetype.isType(Compressed) );
}

QString CompressFilter::destFilename(const KURL &source,
                    const MimeType &sourceType, const MimeType &destType) const
{
  switch (action()) {
  case Extract: {
    QString dest = stripExtension(source, sourceType);
    if ( sourceType.isComposite() )
      return addExtension(dest, KURL(), sourceType.data().baseMimetype);
    return dest;
  }
  case Create:
    return addExtension(source.fileName(), source, destType);
  case Convert: {
    QString dest = stripExtension(source, sourceType);
    if ( sourceType.isComposite() )
      dest = addExtension(dest, KURL(), sourceType.data().baseMimetype);
    return addExtension(dest, KURL(), destType);
  }
  case Check:
    break;
  default:
    Q_ASSERT(false);
    break;
  }
  return QString::null;
}

MimeType CompressFilter::destMimetype(const KURL &source, const MimeType &sourceType,
                                      const MimeType &destType) const
{
  switch (action()) {
  case Extract:
    if ( sourceType.isComposite() ) return sourceType.data().baseMimetype;
    break;
  case Create:
  case Convert:
  case Check:
    break;
  default:
    Q_ASSERT(false);
    break;
  }
  return Filter::destMimetype(source, sourceType, destType);
}

void CompressFilter::fillList(TransformList &list) const
{
  list.append(new CompressKDETransform);
  for (uint i=0; i<CompressProcessTransform::Nb_Types; i++)
    list.append(new CompressProcessTransform(i));
}

bool CompressFilter::isSupported(TransAction action,
                                 const MimeType &mimetype) const
{
  if ( !mimetype.isType(Compressed) ) return false;
  MimeType parent(mimetype.data().parentMimetype);
  // only exact name match: for e.g. 'gzip' can read 'compress' files but not create them
  qDebug("CompressFilter::isSupported %s %s %i %i", mimetype.name().latin1(),
         parent.name().latin1(), action==TransCreate, TransformFilter::isSupported(action, parent)); 
  if ( action==TransCreate && parent!=mimetype ) return false;
  return TransformFilter::isSupported(action, parent);
}

QIODevice *CompressFilter::factory(const MimeType &mimetype,
                                   const QString &source) const
{
  qDebug("compressfilter::factory %s", mimetype.name().latin1());
  GenericTransform *gt = find(TransExtract, mimetype);
  qDebug("%s", gt->className());
  if ( gt==0 ) return 0;
  if ( gt->isA("Konvert::CompressKDETransform") )
    return static_cast<CompressKDETransform *>(gt)->factory(mimetype, source);
  qDebug("trans");
  return static_cast<CompressProcessTransform *>(gt)->factory(mimetype, source);
}

void CompressFilter::addFilters(QPtrList<Filter> &list)
{
  for (uint i=0; i<Nb_Actions; i++)
    list.append(new CompressFilter(Action(i)));
}

#include "compress_filter.moc"
