#include "converter.h"

#include <qdir.h>
#include <qfile.h>
#include <qwidget.h>

#include <klocale.h>
#include <ktempdir.h>
#include <ktempfile.h>
#include <kmessagebox.h>
#include <kio/jobclasses.h>
#include <kio/scheduler.h>
#include <kio/job.h>

using namespace Konvert;

//-----------------------------------------------------------------------------
void FileItem::setURL(const KURL &url)
{
  KFileItem item(KFileItem::Unknown, KFileItem::Unknown, url);
  _url = item.url();
  _mimetype = item.mimeTypePtr();
}

KFileItem *FileItem::item() const
{
  return new KFileItem(KFileItem::Unknown, KFileItem::Unknown, _url, true);
}

//-----------------------------------------------------------------------------
Converter::Converter(Filter &filter, const FileItem &source,
                     const KURL &destDir, const MimeType &destMimetype,
                     QWidget *widget)
  : QObject(widget), _filter(filter), _items(Nb_IOTypes), _destDir(destDir),
    _destMimetype(destMimetype), _job(0), _lister(0), _tmpSrc(0), _tmpDst(0)
{
  _items[Source] = source;
}

QWidget *Converter::widget()
{
  return static_cast<QWidget *>(parent());
}

Converter::~Converter()
{
  qDebug("delete converter");
  _filter.abort();
  delete _tmpSrc;
  delete _tmpDst;
  qDebug("delete converter done");
}

void Converter::setError(Result result, const QString &detailedError)
{
  _error = Filter::message(result, detailedError);
}

void Converter::abort()
{
  if (_job) _job->kill(false);
  if (_lister) _lister->abort();
  _filter.abort();
  setError(Aborted);
}

void Converter::copySource()
{
  qDebug("copy source");
  if ( _items[Source].isLocal() ) {
    // @todo check accessibility
    _items[Source].setLocalURL(_items[Source].url());
    createDestination();
    emit done();
  } else {
    URLType type = _filter.urlType(Source, _items[Source].mimetype());
    _tmpSrc = new TempInode(type, TempInode::DoNotCreate, TempInode::AutoDelete);
    KURL dest;
    dest.setPath(_tmpSrc->name());
    _items[Source].setLocalURL(dest);
    KIO::Scheduler::checkSlaveOnHold(true);
    _job = KIO::file_copy(_items[Source].url(), dest, -1, true, false);
    _job->setWindow(widget());
    connect(_job, SIGNAL(result(KIO::Job *)), SLOT(copySourceDone()));
  }
  qDebug("copy source end");
}

void Converter::copySourceDone()
{
  qDebug("copy source done");
  if ( _job->error() ) setError(ReadingErr);
  else createDestination();
  _job = 0;
  emit done();
}

void Converter::createDestination()
{
  qDebug("create destination");
  URLType type = _filter.urlType(Destination, _items[Destination].mimetype());
  // only create a filename ... (for eg 'arj' has problem with
  // already created destination file).
  TempInode::Mode mode =
    (type==FileURL ? TempInode::DoNotCreate : TempInode::Create);
  _tmpDst = new TempInode(type, mode, TempInode::AutoDelete);
  switch (type) {
    case FileURL: {
      _items[Destination].setURL(_destDir);
      QString filename = _filter.destFilename(_items[Source].url(),
                             _items[Source].mimetype(), _destMimetype);
      KURL url = _items[Destination].url();
      url.addPath(filename);
      _items[Destination].setURL(url);
      break;
    }
    case DirectoryURL:
      _items[Destination].setURL(_destDir);
      break;
    case Nb_URLTypes:
      break;
  }
  KURL url;
  url.setPath(_tmpDst->name());
  _items[Destination].setLocalURL(url);
  qDebug("create dest done file=%s", _items[Destination].url().prettyURL().latin1());
}

void Converter::convert()
{
  qDebug("convert to %s (tmp dest=%s)", _destMimetype.name().latin1(), _items[Destination].localURL().path().latin1());
  QObject::connect(&_filter, SIGNAL(done()), SLOT(convertDoneSlot()));
  _filter.start(_items[Source].localURL().path(), _items[Source].mimetype(),
                _items[Destination].localURL().path(), _destMimetype);
  qDebug("convert done");
}

void Converter::convertDoneSlot()
{
  qDebug("convert done %i (sender=%i)", _filter.result().result, sender());
  _filter.disconnect(this);
  const ResData &res = _filter.result();
  if ( res.result!=Success )
    setError(res.result, res.detailedError);
  emit done();
}

void Converter::copyDestination()
{
  qDebug("copy destination");
  switch ( _filter.urlType(Destination, _items[Destination].mimetype()) ) {
  case Nb_URLTypes:
    emit done();
    break;
  case FileURL:
    qDebug("dest exists=%i", _tmpDst->file()->exists());
    _filesToCopy.append(_items[Destination].localURL());
    copyFiles();
    break;
  case DirectoryURL:
    _lister = new DirLister(this);
    _lister->start(_items[Destination].localURL(), false, widget());
    connect(_lister, SIGNAL(done()), SLOT(listDirDone()));
    break;
  }
}

void Converter::listDirDone()
{
  if ( _lister->hasError() ) setError(WritingErr);
  _filesToCopy = _lister->urls();
  copyFiles();
}

void Converter::copyFiles()
{
  qDebug("copy files to %s", _items[Destination].url().prettyURL().latin1());
  KIO::Scheduler::checkSlaveOnHold(true);
  _job = KIO::copy(_filesToCopy, _items[Destination].url());
  connect(_job, SIGNAL(result(KIO::Job *)), SLOT(copyDestinationDone()));
  _job->setWindow(widget());
}

void Converter::copyDestinationDone()
{
  qDebug("copy dest done: %i", _job->error());
  if ( _job->error() ) setError(WritingErr);
  _job = 0;
  emit done();
}

bool Converter::oneStepConvert()
{
  qDebug("0");
  Waiter w(this, SIGNAL(done()));
  copySource();
  qDebug("00");
  w.wait(); // loop until done
  if ( hasError() ) return false;
  qDebug("1");
  convert();
  w.wait(); // loop until done
  if ( hasError() ) return false;
  qDebug("2");
  copyDestination();
  w.wait(); // loop until done
  qDebug("3");
  return !hasError();
}

#include "converter.moc"
