#include "filter.h"

#include <qregexp.h>
#include <qtimer.h>
#include <qlayout.h>
#include <qlabel.h>

#include <klocale.h>
#include <kio/jobclasses.h>
#include <kio/scheduler.h>
#include <kio/job.h>

using namespace Konvert;

//-----------------------------------------------------------------------------
FilterWidget::FilterWidget(FileType type, Action action)
  : _initialized(false), _type(type), _action(action)
{}

void FilterWidget::init()
{
  if (_initialized) return;
  doInit();
  _initialized = true;
}

MimetypeWidget::MimetypeWidget(FileType type, Action action)
  : FilterWidget(type, action)
{
  QHBoxLayout *top = new QHBoxLayout(this, 10, 10);
  _combo = new KComboBox(this);
  connect(_combo, SIGNAL(activated(int)), SLOT(modifiedSlot()));
  connect(_combo, SIGNAL(activated(int)), SIGNAL(modified()));
  top->addWidget(_combo);
  _extension = new QLabel(this);
  top->addWidget(_extension);
}

void MimetypeWidget::modifiedSlot()
{
  _extension->setText(mimetype().extension());
}

void MimetypeWidget::doInit()
{
  const BasicMimeTypeList &all = MimeType::all(true);
  BasicMimeTypeList::const_iterator it = all.begin();
  for (; it!=all.end(); ++it)
    if ( Filter::isSupported(MimeType::null, type(), action(), it.key()) ) {
      _destMimetypes.insert(it.key());
      _combo->insertItem(it.key().name());
    }
  modifiedSlot();
}

//-----------------------------------------------------------------------------
QPtrList<Filter> *Filter::_list = 0;

Filter *Filter::find(const MimeType &source, FileType type, Action action,
                     const MimeType &dest)
{
  initList();
  QPtrListIterator<Filter> it(*_list);
  for(; it.current()!=0; ++it)
    if ( ( type==Nb_FileTypes || it.current()->type()==type )
         && it.current()->supports(source, action, dest) ) break;
  qDebug("Filter::find %i", it.current());
  return it.current();
}

Filter *Filter::find(const QString &className)
{
  initList();
  QPtrListIterator<Filter> it(*_list);
  for(; it.current()!=0; ++it)
    if ( it.current()->isA(className) ) return it.current();
  return 0;
}

bool Filter::isSupported(const MimeType &mimetype, IOType type)
{
  return isSupported(type==Source ? mimetype : MimeType::null,
                     Nb_FileTypes, Nb_Actions,
                     type==Destination ? mimetype : MimeType::null);
}

bool Filter::isSupported(const MimeType &source, FileType type, Action action,
                         const MimeType &dest)
{
  return find(source, type, action, dest);
}

QString Filter::stripExtension(const KURL &source, const MimeType &sourceType)
{
  QString filename = source.fileName();
  QStringList ext = sourceType.extensions();
  for (uint i=0; i<ext.count(); i++) {
    qDebug("examine pattern: %s", ext[i].latin1());
    QRegExp rg("^(.+)\\." + ext[i] + '$', false);
    if ( rg.exactMatch(filename) ) {
      qDebug("  matching: %s", rg.cap(1).latin1());
      return rg.cap(1);
    }
  }
  return filename;
}

QString Filter::addExtension(const QString &filename,
                 const KURL &source, const MimeType &destType)
{
  QString extension = destType.extension(source);
  if ( !extension.isEmpty() ) return filename + '.' + extension;
  return filename;
}

FilterWidget *Filter::widgetFactory(FileType type, Action action)
{
  switch (action) {
    case Convert:
    case Create:
      return new MimetypeWidget(type, action);
    case Extract:
    case Check:
      return new FilterWidget(type, action);
    case Nb_Actions: Q_ASSERT(false); break;
  }
  return 0;
}

//-----------------------------------------------------------------------------
Filter::Filter(const QString &basename, FileType type, Action action)
  : _name(basename + "_" + FILE_TYPE_DATA[type].name + "_"
          + ACTION_DATA[action].name + "_filter"),
    _type(type), _action(action), _job(0), _supported(Nb_IOTypes)
{}

QString Filter::destFilename(const KURL &source, const MimeType &sourceType,
                             const MimeType &destType) const
{
  return addExtension(stripExtension(source, sourceType), source, destType);
}

MimeType Filter::destMimetype(const KURL &, const MimeType &,
                              const MimeType &destType) const
{
  return destType;
}

const BasicMimeTypeList &Filter::supportedMimetypes(IOType type) const
{
  if ( !_supported[type].initialized ) {
    qDebug("Filter::supportedMimetypes initialize %i", type);
    _supported[type].list = getSupportedMimetypes(type);
    _supported[type].initialized = true;
  }
  return _supported[type].list;
}

bool Filter::supports(const MimeType &source, Action faction,
                     const MimeType &dest) const
{
  if ( faction!=Nb_Actions && faction!=action() ) return false;
  if ( !source.isNull() ) {
    switch (faction) {
    case Create: break;
    case Convert:
    case Check:
    case Extract:
      if ( !source.isType(type()) ) return false;
      break;
    case Nb_Actions: Q_ASSERT(false); break;
    } 
    if ( !supportedMimetypes(Source).contains(source) ) return false;
  }
  if ( dest.isNull() ) return true;
  // source==dest is natively supported by all filters for Convert
  if ( faction==Convert && !source.isNull() && source==dest ) return true;
  qDebug("filter %s %i", dest.name().latin1(), supportedMimetypes(Destination).contains(dest));
  return supportedMimetypes(Destination).contains(dest);
}

void Filter::done(Result result, const QString &detailed)
{
  _result.result = result;
  _result.detailedError = detailed;
  QTimer::singleShot(0, this, SIGNAL(done()));
}

QString Filter::message(Result result, const QString &detailedErr)
{
  QString s = i18n(RESULT_DATA[result].msg);
  if ( RESULT_DATA[result].type==DetailedErrorType )
      s = s.arg(i18n(detailedErr));
  return s;
}

URLType Filter::urlType(IOType type, const MimeType &) const
{
  if ( type==Destination && action()==Check ) return Nb_URLTypes;
  return FileURL;
}

void Filter::start(const QString &source, const MimeType &sourceType,
                   const QString &dest, const MimeType &destType)
{
  if ( action()==Convert && sourceType==destType ) {
    KIO::Scheduler::checkSlaveOnHold(true);
    KURL usource, udest;
    usource.setPath(source);
    udest.setPath(dest);
    _job = KIO::file_copy(usource, udest, -1, true, false);
    connect(_job, SIGNAL(result(KIO::Job *)), SLOT(copyDone()));
  } else startInternal(source, sourceType, dest, destType);
}

void Filter::copyDone()
{
  if ( _job->error() ) done(FilterErr, _job->errorString());
  else done(Success);
  _job = 0;
}

void Filter::abort()
{
  _result.result = Aborted;
  if (_job) _job->kill(false);
  abortInternal();
}

//-----------------------------------------------------------------------------
ProcessFilter::ProcessFilter(const Process::Data &data, FileType type,
                             Action action)
  : Filter(data.command, type, action), _data(data)
{
  _process = new Process(data.command, this);
  connect(_process, SIGNAL(processExited()), SLOT(processExited()));
}

void ProcessFilter::startInternal(const QString &source,
     const MimeType &sourceType, const QString &dest, const MimeType &destType)
{
  QStringList a = args(source, sourceType, dest, destType);
  if ( !_process->run(a, KProcess::NotifyOnExit) )
    done(FilterErr, i18n("Failed to call \"%1\".").arg(_process->command()));
}

void ProcessFilter::abortInternal()
{
  _process->abort();
}

void ProcessFilter::processExited()
{
  qDebug("exit value %i", _process->exitStatus());
  if ( !_process->normalExit() )
    done(FilterErr, i18n("\"%1\" exited anormaly.").arg(_process->command()));
  else if ( _process->exitStatus()==0 ) done(Success);
  else done(FilterErr, i18n("\"%1\" returned exit value: %1")
           .arg(_process->command()).arg(_process->exitStatus()));
}

BasicMimeTypeList ProcessFilter::getSupportedMimetypes(IOType type) const
{
  QStringList args = Process::args(_data.initArgs);
  Process process(_data.command);
  if ( !process.run(args, KProcess::Block, Process::toCom(_data.type)) || !process.normalExit() )
    return BasicMimeTypeList();
  QStringList lines = QStringList::split('\n', process.output(_data.type));
  return findSupportedMimetypes(type, lines);
}

//----------------------------------------------------------------------------
MultiTypeProcessFilter::MultiTypeProcessFilter(const Data &data,
                                               Action action, FileType type)
  : ProcessFilter(data.data, type, action), _data(data)
{}

BasicMimeTypeList MultiTypeProcessFilter::findSupportedMimetypes(IOType type,
                                               const QStringList &lines) const
{
  BasicMimeTypeList mlist;
  QStringList list = extractCodes(type, lines);
  QStringList::const_iterator it = list.begin();
  for (; it!=list.end(); ++it) {
    MimeType mt = findMimetype(type, *it);
    if ( mt.isNull() || !isAccepted(type, mt) ) continue;
    mlist.insert(mt);
  }
  return mlist;
}

MimeType MultiTypeProcessFilter::findMimetype(IOType type,
                                              const QString &code) const
{
  const CodeData *codes = _data.codes[type];
  uint i = 0;
  while (codes[i].code) {
    if ( code==codes[i].code ) {
      if ( codes[i].mimetype ) {
        MimeType mt(codes[i].mimetype);
        if ( !MimeType::all(true).contains(mt) )
          qWarning("%s: unknown mimetype \"%s\".", name().latin1(),
                   mt.name().latin1());
        else return mt;
      }
      return MimeType::null;
    }
    i++;
  }
  if (_data.hasAllCodes)
    qWarning("%s: unknown code \"%s\"", name().latin1(), code.latin1());
  return MimeType::null;
}

QString MultiTypeProcessFilter::findCode(IOType type, const MimeType &mimetype) const
{
  const CodeData *codes = _data.codes[type];
  uint i = 0;
  while (codes[i].code) {
    if ( codes[i].mimetype==mimetype.name() ) return codes[i].code;
    i++;
  }
  Q_ASSERT(false);
  return QString::null;
}

#include "filter.moc"
