#ifndef FILTER_H
#define FILTER_H

#include <qwidget.h>
#include <qptrlist.h>
#include <qvaluevector.h>
#include <qptrvector.h>

#include <kcombobox.h>

#include "mimetype.h"
#include "misc.h"

class KURL;
class QLabel;
namespace KIO {
  class Job;
}

namespace Konvert
{
class FilterList;

//-----------------------------------------------------------------------------
class FilterWidget : public QWidget
{
  Q_OBJECT
public:
   FilterWidget(FileType type, Action action);
   void init();

   FileType type() { return _type; }
   Action action() { return _action; }
   virtual MimeType mimetype() const { return MimeType::null; }

signals:
  void modified();

protected:
  virtual void doInit() {}

private:
  bool           _initialized;
  FileType _type;
  Action         _action;
};

class MimetypeWidget : public FilterWidget
{
  Q_OBJECT
public:
  MimetypeWidget(FileType type, Action action);

  MimeType mimetype() const { return MimeType(_combo->currentText()); }
  const BasicMimeTypeList &destMimetypes() const { return _destMimetypes; }

protected:
  virtual void doInit();

private slots:
  void modifiedSlot();

private:
  BasicMimeTypeList _destMimetypes;
  KComboBox *_combo;
  QLabel    *_extension;
};

//-----------------------------------------------------------------------------
class Filter : public QObject
{
  Q_OBJECT
public:
  Filter(const QString &basename, FileType type, Action action);

  QString name() const { return _name; }
  FileType type() const { return _type; }
  Action action() const { return _action; }
  const BasicMimeTypeList &supportedMimetypes(IOType type) const;
  bool supports(const MimeType &sourceType, Action action,
                const MimeType &destType) const;

  void start(const QString &source, const MimeType &sourceType,
             const QString &dest, const MimeType &destType);
  void abort();
  const ResData &result() const { return _result; }
  void done(Result result, const QString &detailed = QString::null);

  virtual URLType urlType(IOType type, const MimeType &sourceType) const;
  virtual QString destFilename(const KURL &source, const MimeType &sourceType,
                               const MimeType &destType) const;
  virtual MimeType destMimetype(const KURL &source, const MimeType &sourceType,
                               const MimeType &destType) const;

  static Filter *find(const MimeType &source, FileType type,
                      Action action, const MimeType &dest);
  static Filter *find(const QString &className);
  static bool isSupported(const MimeType &mimetype, IOType type);
  static bool isSupported(const MimeType &source,
                          FileType type = Nb_FileTypes,
                          Action action = Nb_Actions,
                          const MimeType &destType = MimeType::null);
  static QString stripExtension(const KURL &source,
                const MimeType &sourceType);
  static QString addExtension(const QString &filename, const KURL &source,
                  const MimeType &deestType);
  static QString message(Result result,
                         const QString &detailedError = QString::null);
  static FilterWidget *widgetFactory(FileType type, Action action);

signals:
  void progress(uint i); // from 1 to 99
  void done();

protected:
  virtual void startInternal(const QString &source, const MimeType &sourceType,
                             const QString &dest, const MimeType &destType) = 0;
  virtual void abortInternal() {}
  virtual BasicMimeTypeList getSupportedMimetypes(IOType) const
    { return BasicMimeTypeList(); }
    
private slots:
  void copyDone();

private:
  QString   _name;
  FileType  _type;
  Action    _action;
  ResData   _result;
  KIO::Job *_job;

  class SupportedData {
  public:
    SupportedData() : initialized(false) {}
    BasicMimeTypeList list;
    bool initialized;
  };
  mutable QValueVector<SupportedData> _supported;

  static QPtrList<Filter> *_list;
  static void initList();

  friend class FiltersDialog;
};

//-----------------------------------------------------------------------------
class ProcessFilter : public Filter
{
  Q_OBJECT
public:
  ProcessFilter(const Process::Data &data, FileType type, Action action);

protected:
  virtual void startInternal(const QString &source, const MimeType &sourceType,
                             const QString &dest, const MimeType &destType);
  virtual void abortInternal();
  virtual QStringList args(const QString &source, const MimeType &sourceType,
                           const QString &dest, const MimeType &destType) const = 0;
  virtual BasicMimeTypeList findSupportedMimetypes(IOType type,
                                                   const QStringList &lines) const = 0;

private slots:
  void processExited();

private:
  const Process::Data &_data;
  Process *_process;

  virtual BasicMimeTypeList getSupportedMimetypes(IOType type) const;
};

//----------------------------------------------------------------------------
class MultiTypeProcessFilter : public ProcessFilter
{
  Q_OBJECT
public:
  struct CodeData {
    const char *code, *mimetype;
  };
  struct Data {
    Process::Data data;
    bool hasAllCodes;
    const CodeData *codes[Nb_IOTypes];
  };
  
  MultiTypeProcessFilter(const Data &data, Action action, FileType type);
  
  virtual BasicMimeTypeList findSupportedMimetypes(IOType type,
                                              const QStringList &lines) const;
              
protected:
  virtual QStringList extractCodes(IOType type,
                                   const QStringList &lines) const = 0;
  virtual bool isAccepted(IOType, const MimeType &) const { return true; }
  QString findCode(IOType type, const MimeType &mimetype) const;
  
private:
  const Data &_data;

  MimeType findMimetype(IOType type, const QString &code) const;
};

} // namespace

#endif
