#include "image_filter.h"

#include <qimage.h>
#include <qregexp.h>

#include <kimageio.h>
#include <klocale.h>

using namespace Konvert;

//----------------------------------------------------------------------------
void Konvert::addImageFilters(QPtrList<Filter> &list)
{
  list.append(new ImageKDEFilter);
  list.append(new GenericConvertFilter(Image));
  list.append(new IdentifyFilter);
}

//----------------------------------------------------------------------------
ImageKDEFilter::ImageKDEFilter()
  : Filter("kimageio", Image, Convert)
{
  KImageIO::registerFormats();
}

BasicMimeTypeList ImageKDEFilter::getSupportedMimetypes(IOType type) const
{
  return KImageIO::mimeTypes(type==Source ? KImageIO::Reading : KImageIO::Writing);
}

void ImageKDEFilter::startInternal(const QString &source, const MimeType &,
                                   const QString &dest, const MimeType &destType)
{
  // read source
  QImageIO ios(source, 0);
  if ( !ios.read() ) {
    done(FilterErr, i18n("File not recognized."));
    return;
  }

  // write dest
  qDebug("image filter dest %s", dest.latin1());
  QString destFormat = KImageIO::typeForMime(destType.name());
  QImageIO iod(dest, destFormat);
  iod.setImage(ios.image());
  if ( !iod.write() ) {
    done(FilterErr, i18n("Conversion/Writing error %1."));
    return;
  }

  done(Success);
}

//----------------------------------------------------------------------------
const MultiTypeProcessFilter::CodeData ImageMagickFilter::CODE_DATA[] = {
  {"8BIM",      0}, // #### no mimetype
  {"8BIMTEXT",  0}, // #### no mimetype
  {"8BIMWTEXT", 0}, // #### no mimetype
  {"AFM",       "application/x-font-afm"},
  {"APP1",      0}, // #### no mimetype
  {"ART",       0}, // #### no mimetype
  {"AVI",       "video/x-msvideo"},
  {"AVS",       0}, // #### no mimetype
  {"BIE",       0}, // same as JBG
  {"BMP",       "image/x-bmp"},
  {"BMP2",      0}, // same as BMP (version 2)
  {"BMP3",      0}, // same as BMP (version 3)
  {"CACHE",     0}, // #### no mimetype
  {"CAPTION",   0}, // #### no mimetype
  {"CMYK",      0}, // #### no mimetype
  {"CMYKA",     0}, // #### no mimetype
  {"CUR",       0}, // same as ICO
  {"CUT",       0}, // #### no mimetype
  {"DCM",       0}, // #### no mimetype
  {"DCX",       0}, // #### no mimetype
  {"DPS",       0}, // #### no mimetype
  {"DPX",       0}, // #### no mimetype
  {"EPDF",      0}, // #### no mimetype
  {"EPI",       0}, // #### no mimetype
  {"EPS",       "image/x-eps"},
  {"EPS2",      0}, // same as EPS (version 2)
  {"EPS3",      0}, // same as EPS (version 3)
  {"EPSF",      0}, // same as EPS
  {"EPSI",      0}, // same as EPS (Interchange format)
  {"EPST",      0}, // same as EPS (with TIFF preview)
  {"EPT",       0}, // same as EPST (typo ?)
  {"EXIF",      0}, // #### no mimetype
  {"FAX",       "image/fax-g3"},
  {"FILE",      0}, // internal
  {"FITS",      0}, // #### no mimetype
  {"FPX",       0}, // #### no mimetype
  {"FRACTAL",   0}, // #### no mimetype
  {"G3",        0}, // same as FAX
  {"GIF",       "image/gif"},
  {"GIF87",     0}, // same as GIF (GIF87 format)
  {"GRADIENT",  0}, // raw
  {"GRANITE",   0}, // internal
  {"GRAY",      0}, // #### no mimetype
  {"H",         0}, // internal
  {"HDF",       0}, // #### no mimetype
  {"HISTOGRAM", 0}, // internal
  {"HTM",       0}, // same as HTML
  {"HTML",      0}, // ???
  {"HTTP",      0}, // internal
  {"ICB",       0}, // same as TARGA
  {"ICC",       0}, // #### no mimetype
  {"ICM",       0}, // same as ICC
  {"ICO",       "image/x-ico"},
  {"ICON",      0}, // same as ICO
  {"IMPLICIT",  0}, // internal
  {"IPTC",      0}, // #### no mimetype
  {"IPTCTEXT",  0}, // #### no mimetype
  {"IPTCWTEXT", 0}, // #### no mimetype
  {"JBG",       0}, // #### no mimetype
  {"JBIG",      0}, // same as JBG
  {"JNG",       "image/x-jng"},
  {"JP2",       "image/jp2"},
  {"JPC",       0}, // #### no mimetype
  {"JPEG",      "image/jpeg",},
  {"JPG",       0}, // same as JPEG
  {"LABEL",     0}, // #### no mimetype
  {"LOGO",      0}, // #### no mimetype
  {"M2V",       0}, // MPEG2 no mimetype ?
  {"MAGICK",    0}, // internal
  {"MAP",       0}, // internal
  {"MAT",       0}, // #### no mimetype
  {"MATTE",     0}, // #### no mimetype
  {"MIFF",      0}, // #### no mimetype
  {"MNG",       "video/x-mng"},
  {"MONO",      0}, // raw
  {"MPC",       0}, // #### no mimetype
  {"MPEG",      "video/mpeg"},
  {"MPG",       0}, // same as MPEG
  {"MPR",       0}, // #### no mimetype
  {"MSL",       0}, // #### no mimetype
  {"MTV",       0}, // #### no mimetype
  {"MVG",       0}, // #### no mimetype
  {"NETSCAPE",  0}, // #### no mimetype
  {"NULL",      0}, // internal
  {"OTB",       0}, // #### no mimetype
  {"P7",        0}, // #### no mimetype
  {"PAL",       0}, // #### no mimetype
  {"PALM",      0}, // #### no mimetype
  {"PATTERN",   0}, // internal
  {"PBM",       "image/x-portable-bitmap"},
  {"PCD",       "image/x-photo-cd"},
  {"PCDS",      0}, // same as PCD
  {"PCL",       0}, // #### no mimetype
  {"PCT",       0}, // same as PICT
  {"PCX",       "image/x-pcx"},
  {"PDB",       0}, // #### no mimetype
  {"PDF",       "application/pdf"},
  {"PFA",       "application/x-font-type1"}, // (ASCII)
  {"PFB",       0}, // same as PFA (BINARY)
  {"PFM",       0}, // same as PFA (?)
  {"PGM",       "image/x-portable-graymap"},
  {"PGX",       0}, // JP2000 VM ???
  {"PICON",     0}, // #### no mimetype
  {"PICT",      0}, // #### no mimetype
  {"PIX",       0}, // #### no mimetype
  {"PLASMA",    0}, // #### no mimetype
  {"PM",        "image/x-xpm"},
  {"PNG",       "image/png"},
  {"PNG24",     0}, // same as PNG (24bit RGB)
  {"PNG32",     0}, // same as PNG (32bit RGBA)
  {"PNG8",      0}, // same as PNG (8bit indexed)
  {"PNM",       "image/x-portable-anymap"},
  {"PPM",       "image/x-portable-pixmap"},
  {"PREVIEW",   0}, // internal
  {"PS",        "application/postscript"},
  {"PS2",       0}, // same as PS (version 2)
  {"PS3",       0}, // same as PS (version 3)
  {"PSD",       "image/x-psd"},
  {"PTIF",      0}, // same as TIFF ???
  {"PWP",       0}, // #### no mimetype
  {"RAS",       "image/x-cmu-raster"},
  {"RGB",       0}, // raw
  {"RGBA",      0}, // raw
  {"RLA",       0}, // #### no mimetype
  {"RLE",       0}, // #### no mimetype
  {"ROSE",      0}, // #### no mimetype
  {"SCT",       0}, // #### no mimetype
  {"SFW",       0}, // #### no mimetype
  {"SGI",       0}, // #### no mimetype
  {"SHTML",     0}, // same as HTML
  {"STEGANO",   0}, // #### no mimetype
  {"SUN",       0}, // same as RAS
  {"SVG",       "image/svg"},
  {"TEXT",      0}, // raw
  {"TGA",       "image/x-targa"},
  {"TIF",       0}, // same as TIFF
  {"TIFF",      "image/tiff"},
  {"TILE",      0}, // internal
  {"TIM",       0}, // #### no mimetype
  {"TTF",       "application/x-font-ttf"},
  {"TXT",       0}, // same as TEXT
  {"UIL",       0}, // #### no mimetype
  {"UYVY",      0}, // raw
  {"VDA",       0}, // same as TGA
  {"VICAR",     0}, // #### no mimetype
  {"VID",       0}, // #### no mimetype
  {"VIFF",      0}, // same as XV
  {"VST",       0}, // same as TGA
  {"WBMP",      0}, // #### no mimetype
  {"WMF",       "image/x-wmf"},
  {"WPG",       0}, // #### no mimetype
  {"X",         0}, // internal
  {"XBM",       "image/x-xbm"},
  {"XC",        0}, // internal
  {"XCF",       "image/x-xcf"},
  {"XML",       0}, // #### no mimetype
  {"XMP",       0}, // #### no mimetype
  {"XPM",       "image/x-xpm"},
  {"XV",        0}, // #### no mimetype
  {"XWD",       "image/x-xwindowdump"},
  {"YUV",       0}, // #### no mimetype
  {0,           0}
};

ImageMagickFilter::ImageMagickFilter(const Data &data, Action action,
                                     FileType type)
  : MultiTypeProcessFilter(data, action, type), _type(type)
{}

QStringList ImageMagickFilter::extractCodes(IOType type,
                                            const QStringList &lines) const
{
  QStringList list;
  QRegExp regExp("^\\s*([A-Z]+)\\*?\\s+([r\\-])([w\\-])([+\\-])\\s+\\w", true);
  QStringList::const_iterator it = lines.begin();
  for (; it!=lines.end(); ++it) {
    if ( regExp.search(*it)!=-1 ) {
      bool read = ( regExp.cap(2)=="r" );
      bool write = ( regExp.cap(3)=="w" );
      //bool multi = ( regExp.cap(4)=="+" );
      if ( (type==Source && read) || (type==Destination && write ) ) 
        list.append(regExp.cap(1));
    }
  }
  return list;
}

bool ImageMagickFilter::isAccepted(IOType type, const MimeType &mimetype) const
{
  if ( type==Source && !mimetype.isType(_type) ) return false;
  return !mimetype.isType(Video); // @fixme to be supported later
}

//----------------------------------------------------------------------------
const MultiTypeProcessFilter::Data GenericConvertFilter::DATA = {
  {"convert", Stdout, "-list", "format", 0}, true, {CODE_DATA, CODE_DATA}
};

GenericConvertFilter::GenericConvertFilter(FileType type)
  : ImageMagickFilter(DATA, Convert, type)
{}

QStringList GenericConvertFilter::args(const QString &source, const MimeType &sourceType,
                                       const QString &dest, const MimeType &destType) const
{
  QStringList args;
  args.append(findCode(Source, sourceType) + ':' + source);
  args.append(findCode(Destination, destType) + ':' + dest);
  return args;
}

//----------------------------------------------------------------------------
const MultiTypeProcessFilter::Data IdentifyFilter::DATA = {
  {"identify", Stdout, "-list", "format", 0}, true, {CODE_DATA, CODE_DATA}
};

IdentifyFilter::IdentifyFilter()
  : ImageMagickFilter(DATA, Check, Image)
{}

QStringList IdentifyFilter::args(const QString &source, const MimeType &sourceType,
                                 const QString &, const MimeType &) const
{
  QStringList args;
  args.append("-format");
  args.append(findCode(Source, sourceType));
  args.append(source);
  return args;
}

#include "image_filter.moc"
