/***************************************************************************
 *   Copyright (C) 2004 by Nicolas Hadacek                                 *
 *   hadacek@kde.org                                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "konvert.h"

#include <qlayout.h>
#include <qtimer.h>
#include <qvgroupbox.h>
#include <qlabel.h>
#include <qpopupmenu.h>
#include <qwidgetstack.h>
#include <qbuttongroup.h>
#include <qlayout.h>

#include <kprogress.h>
#include <kmainwindow.h>
#include <klocale.h>
#include <kfiledialog.h>
#include <kmessagebox.h>
#include <kaction.h>
#include <kstdaction.h>
#include <kiconloader.h>
#include <kdirlister.h>

#include "filter_dialogs.h"

using namespace Konvert;

//----------------------------------------------------------------------------
const char *AddDirectoryDialog::LABELS[Nb_Types] = {
  I18N_NOOP("All Files"), I18N_NOOP("Directories")
};

AddDirectoryDialog::AddDirectoryDialog(QWidget *parent)
  : KDialogBase(Plain, i18n("Choose add directory mode."), Ok|Cancel, Ok,
                parent, "add_directory_dialog", true)
{
  QGridLayout *grid = new QGridLayout(plainPage(), 3, 2, 0, spacingHint());
  
  _recurse = new QCheckBox(i18n("Recursively."), plainPage());
  grid->addMultiCellWidget(_recurse, 0, 0, 0, 1);
  
  grid->addWidget(new QLabel(i18n("Type:"), plainPage()), 1, 0);
  _mimeFilter = new KComboBox(plainPage());
  for (uint i=0; i<Nb_Types; i++)
    _mimeFilter->insertItem(i18n(LABELS[i]));
  for (uint i=0; i<nbSupportedFileTypes(); i++) {
    FileType type = indexToFileType(i);
    _mimeFilter->insertItem(i18n(FILE_TYPE_DATA[type].label));
  }
  grid->addWidget(_mimeFilter, 1, 1);
  
  grid->addWidget(new QLabel(i18n("Filter:"), plainPage()), 2, 0);
  _nameFilter = new QLineEdit(plainPage());
  grid->addWidget(_nameFilter, 2, 1);
}

AddDirectoryDialog::Type AddDirectoryDialog::type() const
{
  if ( _mimeFilter->currentItem()>=Nb_Types ) return Nb_Types;
  return Type(_mimeFilter->currentItem());
}

FileType AddDirectoryDialog::fileType() const
{
  if ( type()==Directories ) return Directory;
  if ( type()==Nb_Types ) 
    return indexToFileType(_mimeFilter->currentItem()-Nb_Types);
  return Nb_FileTypes;
}

KURL::List AddDirectoryDialog::getURLs(const KURL &dir)
{
  DirLister lister;
  if ( fileType()!=Directory ) lister.setExclusions(DirLister::Directories);
  lister.setNameFilter(_nameFilter->text());
  lister.setMimeFilter(fileType());
  Waiter w(&lister, SIGNAL(done()));
  lister.start(dir, _recurse->isChecked(), this);
  w.wait(); // loop until done
  if ( lister.hasError() ) return KURL::List();
  return lister.urls();
}

//----------------------------------------------------------------------------
MainWindow::MainWindow()
  : KMainWindow(0, "konvert"), _filtersDialog(0), _mimetypesDialog(0),
    _addDirDialog(0), _converter(0)
{
  setFocusPolicy(StrongFocus);

// GUI
  QWidget *w = new QWidget(this);
  setCentralWidget(w);
  QVBoxLayout *top = new QVBoxLayout(w, 10, 10);

  // listview
  QVGroupBox *gbox = new QVGroupBox(i18n("Todo List"), w);
  top->addWidget(gbox);
  QWidget *widget = new QWidget(gbox);
  QVBoxLayout *vbox = new QVBoxLayout(widget, 0, 10);

  _todo = new TodoListView(widget);
  connect(_todo, SIGNAL(changed()), SLOT(updateStartButton()));
  vbox->addWidget(_todo);

  QHBoxLayout *hbox= new QHBoxLayout(vbox);
  _add = new KPushButton(i18n("Add"), widget);
  QPopupMenu *popup = new QPopupMenu(_add);
  popup->insertItem(i18n("Add Files..."), this, SLOT(addFiles()));
  popup->insertItem(i18n("Add Directory..."), this, SLOT(addDirectory()));
  popup->insertItem(i18n("Add Files in Directory..."), this, SLOT(addFilesDir()));
  _add->setPopup(popup);
  hbox->addWidget(_add);
  _clearList = new KPushButton(KGuiItem(i18n("Clear All"), "remove"), widget);
  connect(_clearList, SIGNAL(clicked()), SLOT(clearList()));
  hbox->addWidget(_clearList);
  _clearSuccess = new KPushButton(KGuiItem(i18n("Clear Processed"), "remove"), widget);
  connect(_clearSuccess, SIGNAL(clicked()), SLOT(clearSuccess()));
  hbox->addWidget(_clearSuccess);

  // Filter
  gbox = new QVGroupBox(i18n("Filter"), w);
  top->addWidget(gbox);
  widget = new QWidget(gbox);
  vbox = new QVBoxLayout(widget, 0, 10);

  QGridLayout *grid = new QGridLayout(vbox, 1, 3);
  _typeCombo = new KComboBox(widget);
  connect(_typeCombo, SIGNAL(activated(int)), SLOT(typeChanged()));
  grid->addWidget(_typeCombo, 0, 0);
  _actionCombo = new KComboBox(widget);
  connect(_actionCombo, SIGNAL(activated(int)), SLOT(actionChanged()));
  grid->addWidget(_actionCombo, 0, 1);

  hbox = new QHBoxLayout(10);
  grid->addLayout(hbox, 0, 2);
  _stack = new QWidgetStack(widget);
  hbox->addWidget(_stack);

  hbox = new QHBoxLayout(vbox);
  QLabel *label = new QLabel(i18n("Destination:"), widget);
  hbox->addWidget(label);
  _destDir = new KURLRequester(widget);
  _destDir->setMode(KFile::Directory | KFile::ExistingOnly);
  connect(_destDir, SIGNAL(textChanged(const QString &)),
      SLOT(initAll()));
  hbox->addWidget(_destDir);

  hbox = new QHBoxLayout(vbox);
  _start = new KPushButton(KGuiItem(i18n("Start"), "start"), widget);
  connect(_start, SIGNAL(clicked()), SLOT(start()));
  hbox->addWidget(_start);
  _abort = new KPushButton(KGuiItem(i18n("Abort"), "cancel"), widget);
  connect(_abort, SIGNAL(clicked()), SLOT(abort()));
  hbox->addWidget(_abort);
  _totalProgress = new KProgress(widget);
  hbox->addWidget(_totalProgress);

  // menu & co
  (void)new KAction(KGuiItem(i18n("Show Filters")), 0,
                    this, SLOT(showFilters()),
                    actionCollection(), "file_show_filters");
  (void)new KAction(KGuiItem(i18n("Show Mimetypes")), 0,
                    this, SLOT(showMimetypes()),
                    actionCollection(), "file_show_mimetypes");
  KStdAction::quit(this, SLOT(close()), actionCollection());
  createGUI();

  // init
  for (uint i=0; i<nbSupportedFileTypes(); i++) {
    FileType type = indexToFileType(i);
    if ( Filter::isSupported(MimeType::null, type) )
      _typeCombo->insertItem(i18n(FILE_TYPE_DATA[type].label));
    for (uint j=0; j<Nb_Actions; j++)
      _stack->addWidget(Filter::widgetFactory(type, Action(j)), type * Nb_FileTypes + j);
  }
  setState(Stopped, false);
  typeChanged();
}

MainWindow::~MainWindow()
{
  delete _converter;
}

void MainWindow::addFiles()
{
  addItems( KFileDialog::getOpenURLs(QString::null, QString::null, this) );
}

void MainWindow::addDirectory()
{
  QString dir = KFileDialog::getExistingDirectory(QString::null, this);
  if ( dir.isEmpty() ) return;
  KURL url;
  url.setDirectory(dir);
  addItems(url);
}

void MainWindow::addFilesDir()
{
  if ( _addDirDialog==0 ) _addDirDialog = new AddDirectoryDialog(this);
  int res = _addDirDialog->exec();
  if ( res==QDialog::Rejected ) return;
  QString dir = KFileDialog::getExistingDirectory(QString::null, this);
  if ( dir.isEmpty() ) return;
  KURL url;
  url.setDirectory(dir);
  addItems( _addDirDialog->getURLs(url) );
}

void MainWindow::addItems(const KURL::List &urls)
{
  for (uint i=0; i<urls.count(); i++) addItem(urls[i]);
  _totalProgress->setProgress(0);
}

void MainWindow::addItem(const KURL &url)
{
  TodoListViewItem *item = _todo->addItem(url);
  if (item) init(item);
}

void MainWindow::clearList()
{
  _todo->clear();
  _totalProgress->setProgress(0);
}

void MainWindow::clearSuccess()
{
  TodoListViewItem *item =
    static_cast<TodoListViewItem *>(_todo->firstChild());
  while (item) {
    TodoListViewItem *prev = item;
    item = static_cast<TodoListViewItem *>(item->nextSibling());
    if ( prev->state()==TodoListViewItem::Success ) _todo->removeItem(prev);
  }
  _totalProgress->setProgress(0);
}

void MainWindow::init(TodoListViewItem *item)
{
  KURL source = item->source().url();
  MimeType sourceType = item->source().mimetype();
  Data &data = _sourceMimetypes[sourceType];
  if ( !data.initDone ) {
    data.filter = Filter::find(sourceType, type(), action(), destMimetype());
    data.initDone = true;
  }
  QString destFilename, destType, message;
  TodoListViewItem::State state = TodoListViewItem::Normal;
  if (data.filter) {
    destFilename = data.filter->destFilename(source, sourceType, destMimetype());
    if ( data.filter->urlType(Destination, destMimetype())==Nb_URLTypes )
      destType = i18n("n/a");
    else {
      MimeType dm = data.filter->destMimetype(source, sourceType, destMimetype());
      destType = (dm.isNull() ? i18n("unknown") : dm.name());
      item->setDataLoss(dm.hasProperty(MimeType::Lossy));
    }
  } else {
    state = TodoListViewItem::NoFilter;
    message = Filter::message(NoFilterErr);
  }
  item->setState(state);
  item->setText(TodoListView::Destination, destFilename);
  item->setText(TodoListView::DestMimetype, destType);
  item->setText(TodoListView::Message, message);
}

void MainWindow::initAll()
{
  qDebug("init all");
  _totalProgress->setProgress(0);
  _sourceMimetypes.clear();
  TodoListViewItem *item =
    static_cast<TodoListViewItem *>(_todo->firstChild());
  while (item) {
    init(item);
    item = static_cast<TodoListViewItem *>(item->nextSibling());
  }
}

FileType MainWindow::type() const
{
  return indexToFileType(_typeCombo->currentItem());
}

Action MainWindow::action() const
{
  return Action(_actionCombo->currentItem());
}

MimeType MainWindow::destMimetype() const
{
  return widget()->mimetype();
}

FilterWidget *MainWindow::widget() const
{
  uint id = type() * Nb_FileTypes + action();
  return static_cast<FilterWidget *>(_stack->widget(id));
}

void MainWindow::setState(State state, bool proc)
{
  qDebug("set state %i", state);
  _state = state;
  bool running = ( state!=Stopped );
  _add->setEnabled(!running);
  _clearList->setEnabled(!running);
  _clearSuccess->setEnabled(!running);
  _todo->setEnabled(!running);
  _abort->setEnabled(running);
  updateStartButton();
  if (proc) process();
}

void MainWindow::updateStartButton()
{
  _start->setEnabled( _todo->childCount() && _state==Stopped );
}

void MainWindow::start()
{
  qDebug("start");
  if ( destDir().isEmpty() ) {
    KMessageBox::sorry(this, i18n("You should specify a destination directory."));
    return;
  }
  _step = 0;
  _totalProgress->setTotalSteps(Nb_States * _todo->childCount());
  _current = static_cast<TodoListViewItem *>(_todo->firstChild());
  setState(Init, true);
  qDebug("start done");
}

void MainWindow::abort()
{
  qDebug("abort !!!!");
  if (_converter) _converter->abort();
  setState(Stopped, true);
}

void MainWindow::stop()
{
  if ( _current==0 ) return;
  _current->setProgress(0);
  if ( _converter==0 ) return;
  if ( _converter->hasError() ) {
    _current->setText(TodoListView::Message, _converter->error());
    _current->setState(TodoListViewItem::Error);
  } else {
    _current->setText(TodoListView::Message, i18n("Success"));
    _current->setState(TodoListViewItem::Success);
    const FileItem &dest = _converter->item(Destination);
    _current->setPixmap(TodoListView::Destination, dest.pixmap());
    _current->setText(TodoListView::DestMimetype, dest.mimetype().name());
  }
  delete _converter;
  _converter = 0;
}

void MainWindow::process()
{
  qDebug("process state=%i", _state);
  _totalProgress->setProgress(Nb_States*_step + _state);
  if (_current) _current->setProgress((100*_state) / (Nb_States-1));
  switch (_state) {
    case Init: {
      _todo->ensureItemVisible(_current);
      _success = false;
      _step++;
      MimeType mimetype = _current->source().mimetype();
      const Data &data = _sourceMimetypes[mimetype];
      if ( data.filter==0 ) setState(Done, true);
      else {
        _converter = new Converter(*data.filter, _current->source(),
                                   destDir(),  destMimetype(), this);
        connect(_converter, SIGNAL(done()), SLOT(process()));
        setState(Convert, false);
        _converter->copySource();
      }
      break;
    }
    case Convert:
      if ( !_converter->hasError() ) {
        setState(CopyDest, false);
        _converter->convert();
      } else setState(Done, true);
      break;
    case CopyDest:
      if ( !_converter->hasError() ) {
        setState(Done, false);
        _converter->copyDestination();
      } else setState(Done, true);
      break;
    case Done:
      stop();
      _current = static_cast<TodoListViewItem *>(_current->nextSibling());
      if (_current) setState(Init, true);
      else {
        _totalProgress->setProgress(Nb_States * _step);
        setState(Stopped, false);
      }
      break;
    case Stopped:
      stop();
      break;
  }
  qDebug("process done");
}

void MainWindow::typeChanged()
{
  _actionCombo->clear();
  for (uint i=0; i<Nb_Actions; i++)
    if ( Filter::isSupported(MimeType::null, type(), Action(i)) )
      _actionCombo->insertItem(i18n(ACTION_DATA[i].label));
  actionChanged();
}

void MainWindow::actionChanged()
{
  if (_stack->visibleWidget())
    _stack->visibleWidget()->disconnect(SIGNAL(modified()));
  widget()->init();
  _stack->raiseWidget(widget());
  connect(widget(), SIGNAL(modified()), SLOT(initAll()));
  initAll();
}

void MainWindow::showFilters()
{
  if ( _filtersDialog==0 ) _filtersDialog = new FiltersDialog(this);
  _filtersDialog->show();
}

void MainWindow::showMimetypes()
{
  if ( _mimetypesDialog==0 ) _mimetypesDialog = new MimetypesDialog(this);
  _mimetypesDialog->show();
}

bool MainWindow::queryClose()
{
  if ( !isRunning() ) return true;
  int res =
    KMessageBox::warningYesNo(this, i18n("Work in progress; quit anyway?"));
  return ( res==KMessageBox::Yes );
}

bool MainWindow::queryExit()
{
  if ( isRunning() ) abort();
  return true;
}

#include "konvert.moc"
