/***************************************************************************
 *   Copyright (C) 2004 by Nicolas Hadacek                                 *
 *   hadacek@kde.org                                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <kapplication.h>
#include <kaboutdata.h>
#include <kcmdlineargs.h>
#include <klocale.h>
#include <kstaticdeleter.h>

#include "konvert.h"
#include "image_filter.h"
#include "compress_filter.h"
#include "archive_filter.h"
#include "audio_filter.h"
#include "publish_filter.h"

using namespace Konvert;

//-----------------------------------------------------------------------------
static KStaticDeleter<QPtrList<Filter> > fsd;

void Filter::initList()
{
  if (_list) return;
  fsd.setObject(_list, new QPtrList<Filter>);
  _list->setAutoDelete(true);

  addImageFilters(*_list);
  ArchiveFilter::addFilters(*_list);
  CompressFilter::addFilters(*_list);
  addAudioFilters(*_list);
  addPublishingFilters(*_list);
}

//-----------------------------------------------------------------------------
static const char description[] =
    I18N_NOOP("Konvert is an application to batch-convert files.");

static const char version[] = "0.8";

static KCmdLineOptions options[] =
{
  { "t", 0, 0 },
  { "type <type>", I18N_NOOP("File type."), 0 }, // @todo add list of valid types ?
  { "a", 0, 0 },
  { "action <action>", I18N_NOOP("Action to be executed."), 0 },
  { "o", 0, 0 },
  { "options <options>", I18N_NOOP("Options for the action (usually a mimetype)."), 0 },
  { "d", 0, 0 },
  { "dest <url>", I18N_NOOP("Destination directory (path or url)."), 0 },
  { "+source", I18N_NOOP("Source file or directory (path or url)."), 0 },
  KCmdLineLastOption
};

KURL decodeURL(const QString &s)
{
  KURL url = KURL::fromPathOrURL(s);
  if ( !url.protocol().isNull() ) return url;
  // we have a relative unix path (or a malformed url...)
  url = KURL();
  QFileInfo info(s);
  url.setPath(info.absFilePath());
  return url;
}

namespace Konvert 
{
enum CmdResultType { SuccessConvert = 0, UnknownFileType, FileTypeNotSupported,
  UnknownAction, UnknownDestMimetype, ActionNotSupported, ActionNeedsMimetype,
  DestMimetypeNotSupported, InvalidDestURL, InvalidSourceURL,
  CannotReadSource, FilterNotFound, FilterError, Nb_CmdResultTypes };
struct CmdResultData {
  int code;
  const char *message;
};
const CmdResultData CMD_RESULT_DATA[Nb_CmdResultTypes] = {
  { 0,  0                                                      }, // Success
  { 1,  I18N_NOOP("Unknown file type.")                        },
  { 2,  I18N_NOOP("File type not supported.")                  },
  { 3,  I18N_NOOP("Unknown action.")                           },
  { 4,  I18N_NOOP("Unknown destination mimetype.")             },
  { 5,  I18N_NOOP("Action not supported for given file type.") },
  { 6,  I18N_NOOP("Action needs a mimetype as option.")        },
  { 7,  I18N_NOOP("Destination mimetype not supported for given action.") },
  { 8,  I18N_NOOP("Invalid path or url for destination directory.") },
  { 9,  I18N_NOOP("Invalid path or url for source file.")      },
  { 10, I18N_NOOP("Cannot read source file.")                  },
  { 11, I18N_NOOP("No filter found.")                          },
  { 12, 0                                                      } // filter error
};
} // namespace

CmdResultType convert(const QString &type, const QString &action,
                 const QString &options, const KURL &dest, const KURL &source,
                 QString &error)
{
  // check file type
  FileType fileType = Nb_FileTypes;
  bool unknown = true;
  for (uint i=0; i<Nb_FileTypes; i++)
    if ( type==FILE_TYPE_DATA[i].name ) {
      unknown = false;
      if ( Filter::isSupported(MimeType::null, FileType(i)) ) {
        fileType = FileType(i);
        break;
      }
    }
  if (unknown ) return UnknownFileType;
  if ( fileType==Nb_FileTypes ) return FileTypeNotSupported;

  // check action
  Action actionType = Nb_Actions;
  unknown = true;
  for (uint i=0; i<Nb_Actions; i++)
    if ( action==ACTION_DATA[i].name ) {
      unknown = false;
      if ( Filter::isSupported(MimeType::null, fileType, Action(i)) ) {
        actionType = Action(i);
        break;
      }
    }
  if (unknown) return UnknownAction;
  if ( actionType==Nb_Actions ) return ActionNotSupported;

  // check options
  MimeType destMimetype;
  if ( ACTION_DATA[actionType].needMimetype ) {
    if ( options.isEmpty() ) return ActionNeedsMimetype;
    destMimetype = MimeType(options);
    if ( !destMimetype.isKnown() ) return UnknownDestMimetype;
    if ( !Filter::isSupported(MimeType::null, fileType, actionType, destMimetype) )
      return DestMimetypeNotSupported;
  }

  // check destination
  if ( !dest.isValid() ) return InvalidDestURL;

  // check source
  if ( !source.isValid() ) return InvalidSourceURL;

  // find filter
  FileItem fsource(source);
  if ( !fsource.item()->isReadable() ) return CannotReadSource;
  qDebug("main: find filter");
  Filter *filter = Filter::find(fsource.mimetype(), fileType, actionType, destMimetype);
  if ( filter==0 ) return FilterNotFound;

  // do convertion
  Converter converter(*filter, fsource, dest, destMimetype, 0);
  if ( converter.oneStepConvert() ) return SuccessConvert;
  error = converter.error();
  return FilterError;
}

int main(int argc, char **argv)
{
    KAboutData about("konvert", I18N_NOOP("Konvert"), version, description,
                     KAboutData::License_GPL, "(C) 2004 Nicolas Hadacek",
		     0, 0, "hadacek@kde.org");
    about.addAuthor("Nicolas Hadacek", 0, "hadacek@kde.org");
    KCmdLineArgs::init(argc, argv, &about);
    KCmdLineArgs::addCmdLineOptions(options);
    KApplication app;

    if ( app.isRestored() ) {
        RESTORE(MainWindow);
    } else {
        KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
        if ( args->count()==0 ) {
          MainWindow *win = new MainWindow();
          app.setMainWidget(win);
          win->show();
        } else {
          QString type = args->getOption("type");
          QString action = args->getOption("action");
          QString options = args->getOption("options");
          KURL dest = decodeURL(args->getOption("dest"));
          KURL source = decodeURL(args->arg(0));
          QString error;
          CmdResultType res = convert(type, action, options, dest, source, error);
          if ( res!=SuccessConvert ) {
            QString msg =
              (res==FilterError ? error : QString(CMD_RESULT_DATA[res].message));
            qDebug("%s", msg.latin1());
          }
          return CMD_RESULT_DATA[res].code;
        }
        args->clear();
    }

    return app.exec();
}

