#include "mimetype.h"

#include <qfileinfo.h>

#include <kstaticdeleter.h>
#include <klocale.h>

using namespace Konvert;

//-----------------------------------------------------------------------------
const FileTypeData Konvert::FILE_TYPE_DATA[Nb_FileTypes] = {
  { 0,            0                        }, // unknown
  { 0,            0                        }, // alias
  { 0,            0                        }, // internal
  { 0,            0                        }, // directory
  { "compressed", I18N_NOOP("Compressed")  },
  { "archive",    I18N_NOOP("Archive")     },
  { "image",      I18N_NOOP("Image")       },
  { "audio",      I18N_NOOP("Audio")       },
  { 0,            I18N_NOOP("Video")       }, // @fixme
  { 0,            I18N_NOOP("Document")    }, // @fixme
  { 0,            I18N_NOOP("Spreadsheet") }, // @fixme
  { 0,            I18N_NOOP("Equation")    }, // @fixme
  { 0,            I18N_NOOP("Graph")       }, // @fixme
  { 0,            I18N_NOOP("Code")        }, // @fixme
  { 0,            I18N_NOOP("Finance")     }, // @fixme
  { 0,            I18N_NOOP("Package")     }, // @fixme
  { 0,            I18N_NOOP("Database")    }, // @fixme
  { 0,            I18N_NOOP("Font")        }, // @fixme
  { "publishing", I18N_NOOP("Publishing")  }
};

uint Konvert::nbSupportedFileTypes()
{
  uint nb = 0;
  for (uint i=0; i<Nb_FileTypes; i++)
    if (FILE_TYPE_DATA[i].name) nb++;
  return nb;
}

FileType Konvert::indexToFileType(uint k)
{
  for (uint i=0; i<Nb_FileTypes; i++) {
    if ( FILE_TYPE_DATA[i].name==0 ) continue;
    if ( k==0 ) return FileType(i);
    k--;
  }
  Q_ASSERT(false);
  return Nb_FileTypes;
}

uint Konvert::toIndex(FileType type)
{
  Q_ASSERT(FILE_TYPE_DATA[type].name);
  uint k = 0;
  for (int i=0; i<type-1; i++) {
    if ( FILE_TYPE_DATA[i].name==0 ) continue;
    k++;
  }
  return k;
}

static KStaticDeleter<BasicMimeTypeList> allsd;
BasicMimeTypeList *MimeType::_all = 0;

static KStaticDeleter<BasicMimeTypeList> allwasd;
BasicMimeTypeList *MimeType::_allwa = 0;

static KStaticDeleter<QMap<QString, const MimeType::Data *> > mimesd;
QMap<QString, const MimeType::Data *> *MimeType::_mimetypes = 0;

static KStaticDeleter<QValueVector<BasicMimeTypeList> > alltypedd;
QValueVector<BasicMimeTypeList> *MimeType::_allTyped = 0;

const MimeType::Data MimeType::NullData = { Unspecified, Unknown, 0, 0, 0 };
const MimeType::Data MimeType::UnspecifiedData = { Unspecified, None, 0, 0, 0 };
const MimeType MimeType::null;

const BasicMimeTypeList &MimeType::all(bool withoutAlias)
{
  initAll();
  return (withoutAlias ? *_allwa : *_all);
}

const BasicMimeTypeList &MimeType::all(FileType type)
{
  initAll();
  return _allTyped->at(type);
}

void MimeType::initMimetypes()
{
  if (_mimetypes) return;
  mimesd.setObject(_mimetypes, new QMap<QString, const Data *>);
  for (uint i=0; DATA[i].mimetype!=0; i++)
    _mimetypes->insert(DATA[i].mimetype, &DATA[i].data);
}

void MimeType::initAll()
{
  if (_all) return;
  allsd.setObject(_all, new BasicMimeTypeList);
  allwasd.setObject(_allwa, new BasicMimeTypeList);
  QValueList<MimeType> list = allMimeTypes();
  QValueList<MimeType>::const_iterator it = list.begin();
  for (; it!=list.end(); ++it) {
    if ( (*it).data().type==Internal ) continue;
    _all->insert(*it);
    if ( (*it).isAlias() ) continue;
    _allwa->insert(*it);
  }
  
  // checks
  BasicMimeTypeList::const_iterator itb = _all->begin();
  for (; itb!=_all->end(); ++itb) {
    QString name = itb.key().name();
    if ( !isKnown(name) )
      qWarning("mimetype %s is unknown !", name.latin1());
    const Data &d = data(name);
    if ( d.type==Archive && d.baseMimetype==0 )
      qWarning("unspecified base mimetype for %s", name.latin1());
    if ( d.baseMimetype && !isKnown(d.baseMimetype) )
      qWarning("base mimetype unknown for %s", name.latin1());
    if ( d.type==Compressed && d.parentMimetype==0 )
      qWarning("unspecified compress mimetype for %s", name.latin1());
    if ( d.type==Alias ) {
      if ( d.parentMimetype==0 )
        qWarning("parent not specified for alias %s", name.latin1());
      if ( d.extensions)
        qWarning("Extensions specified for alias %s ... should be in parent", name.latin1());
    }
    if ( d.parentMimetype && !isKnown(d.parentMimetype) )
      qWarning("parent mimetype unknown for %s", name.latin1());
  }
  
  alltypedd.setObject(_allTyped, new QValueVector<BasicMimeTypeList>(Nb_FileTypes));
  BasicMimeTypeList::const_iterator ita = _all->begin();
  for (; ita!=_all->end(); ++ita) {
    for (uint i=0; i<Nb_FileTypes; i++)
      if ( ita.key().isType(FileType(i)) ) _allTyped->at(i).insert(ita.key());
  }
}

QValueList<MimeType> MimeType::allMimeTypes()
{
  QValueList<MimeType> res;
  KMimeType::List list = KMimeType::allMimeTypes();
  KMimeType::List::iterator it = list.begin();
  for (; it!=list.end(); ++it) {
    QString primary = (*it)->name().section('/', 0, 0);
    if ( primary=="all" || primary=="fonts"
         || primary=="kdedevice" || primary=="message"
         || primary=="multipart"
         || primary=="inode" || primary=="print" ) continue;
    res.append((*it)->name());
  }
  res.append("inode/directory");
  return res;
}

QString MimeType::unalias(const QString &mimetype)
{
  initMimetypes();
  uint i = 0;
  QString name = mimetype;
  for (;;) {
    const Data &d = data(name);
    if ( d.type!=Alias ) return name;
    name = d.parentMimetype;
    i++;
    if ( i==10 ) break;
  }
  qFatal("infinite loop in mimetype alias %s", name.latin1());
  return QString::null;
}

bool MimeType::isKnown(const QString &mimetype)
{
  initMimetypes();
  return _mimetypes->contains(mimetype);
}

const MimeType::Data &MimeType::data(const QString &mimetype)
{
  initMimetypes();
  QMap<QString, const Data *>::const_iterator itm = _mimetypes->find(mimetype);
  if ( itm==_mimetypes->end() ) {
    qDebug("data ask for unfound mimetype \"%s\"", mimetype.latin1());
    return NullData;
  }
  return *itm.data();
}

//-----------------------------------------------------------------------------
void BasicMimeTypeList::insert(const QStringList &mimetypes)
{
  QStringList::const_iterator it = mimetypes.begin();
  for (; it!=mimetypes.end(); ++it) insert(MimeType(*it));
}

//-----------------------------------------------------------------------------
void MimeType::init(const QString &name, bool noUnalias)
{
  _aliasName = name;
  _name = (noUnalias ? name : unalias(name));
  _data = &data(_name);
}

MimeType::MimeType(const KMimeType::Ptr &mime, bool noUnalias)
{
  init(mime->name(), noUnalias);
}

MimeType::MimeType(const QString &name, bool noUnalias)
{
  init(name, noUnalias);
}

MimeType::MimeType(const char *name, bool noUnalias)
{
  init(name, noUnalias);
}

MimeType::MimeType(const QString &primary, const QString &secondary,
                   bool noUnalias)
{
  init(primary + '/' + secondary, noUnalias);
}

QString MimeType::name(bool alias) const
{
  return (alias ? _aliasName : _name);
}

QString MimeType::primary(bool alias) const
{
  return name(alias).section('/', 0, 0);
}

QString MimeType::secondary(bool alias) const
{
  return name(alias).section('/', 1, 1);
}

QString MimeType::extension(const KURL &url) const
{
  QStringList ext = extensions();
  if ( ext.count()!=0 ) return ext[0];
  QFileInfo info( url.fileName() );
  return info.extension();
}

QStringList MimeType::extensions() const
{
  QStringList ext;
  // kde extensions
  QStringList patterns = ptr()->patterns();
  for (uint i=0; i<patterns.count(); i++)
    if ( patterns[i].length()>2 && patterns[i][0]=='*'
         && patterns[i][1]=='.' )
      ext.append(patterns[i].mid(2));
  // additionnal extensions
  if (_data->extensions)
    ext += QStringList::split(' ', _data->extensions);
  return ext;
}

QPixmap MimeType::pixmap(bool enabled) const
{
  return ptr()->pixmap(KIcon::Small, 0,
                       enabled ? KIcon::DefaultState : KIcon::DisabledState);
}

bool MimeType::isType(FileType type) const
{
  if ( data().type==type ) return true;
  if ( data().parentMimetype==0 ) return false;
  MimeType parent(data().parentMimetype);
  return ( parent!=(*this) && parent.isType(type) );
}

bool MimeType::isComposite() const
{
  return ( data().baseMimetype && data().parentMimetype
           && MimeType(data().parentMimetype).data().type==Compressed );
}
