/***************************************************************************
 *   Copyright (C) 2004 by Nicolas Hadacek                                 *
 *   hadacek@kde.org                                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef MIMETYPE_H
#define MIMETYPE_H

#include <qvaluevector.h>

#include <kmimetype.h>

namespace Konvert
{

class MimeType;

//-----------------------------------------------------------------------------
template <class Type>
class MimeTypeList : public QMap<MimeType, Type>
{
public:
  MimeTypeList() {}
  MimeTypeList(const MimeType &mimetype, Type type)
    { insert(mimetype, type); }

  typedef QMapConstIterator<MimeType, Type> const_iterator;

  QStringList primaryTypes() const {
    QStringList list;
    for (const_iterator it = this->begin(); it!=this->end(); ++it) {
      if ( list.find(it.key().primary())!=list.end() ) continue;
      list.append(it.key().primary());
    }
    list.sort();
    return list;
  }

  QStringList secondaryTypes(const QString &primary) const {
    QStringList list;
    for (const_iterator it = this->begin(); it!=this->end(); ++it) {
      if ( it.key().primary()!=primary ) continue;
      list.append(it.key().secondary());
    }
    list.sort();
    return list;
  }
  
  QStringList names() const {
    QStringList list;
    for (const_iterator it = this->begin(); it!=this->end(); ++it)
      list.append(it.key().name());
    return list;
  }
};

class BasicMimeTypeList : public MimeTypeList<void *>
{
public:
  BasicMimeTypeList() {}
  BasicMimeTypeList(const MimeType &mimetype) { insert(mimetype); }
  BasicMimeTypeList(const QStringList &mimetypes) { insert(mimetypes); }

  void insert(const MimeType &mimetype)
    { MimeTypeList<void *>::insert(mimetype, 0); }
  void insert(const QStringList &mimetypes);
};

//-----------------------------------------------------------------------------
enum FileType { Unspecified = 0, Alias, Internal, Directory,
                Compressed, Archive, Image, Audio, Video,
                Document, Spreadsheet, Equation, Graph, Code, Finance,
                Package, Database, Font, Publishing,
                Nb_FileTypes};
struct FileTypeData {
  const char *name, *label;
};
extern const FileTypeData FILE_TYPE_DATA[Nb_FileTypes];
uint nbSupportedFileTypes();
FileType indexToFileType(uint i);
uint toIndex(FileType type);

class MimeType
{
public:
  enum Property { Unknown = -1, None = 0, Template = 1, Lossy = 2, Vectorial = 4,
                  MultiContent = 8 };
  struct Data {
    FileType type;
    int property;
    const char *baseMimetype; // base mimetype for compressed files
    const char *parentMimetype; // mimetype for compression, alias, or parent type
    const char *extensions; // additionnal extensions separated by space
  };

public:
  MimeType() : _data(&NullData) {}
  MimeType(const KMimeType::Ptr &mime, bool noUnalias = false);
  MimeType(const QString &name, bool noUnalias = false);
  MimeType(const char *name, bool noUnalias = false);
  MimeType(const QString &primary, const QString &secondary,
           bool noUnalias = false);

  bool isNull() const { return _name.isEmpty(); }
  QString name(bool alias = false) const;
  QString primary(bool alias = false) const;
  QString secondary(bool alias = false) const;
  // if mimetype has no extension, take the one from the given url
  QString extension(const KURL &url = KURL()) const;
  QStringList extensions() const;
  QPixmap pixmap(bool enabled) const;
  const Data &data() const { return *_data; }
  bool isType(FileType type) const;
  bool hasProperty(Property property) const { return data().property & property; }
  bool isAlias() const { return data().type==Alias; }
  bool isDirectory() const { return data().type==Directory; }
  bool isComposite() const;
  bool isKnown() const { return data().property!=Unknown; }

  bool operator ==(const MimeType &mimetype) const { return name()==mimetype.name(); }
  bool operator !=(const MimeType &mimetype) const { return name()!=mimetype.name(); }
  bool operator ==(const QString &mimetype) const { return name()==mimetype; }
  bool operator !=(const QString &mimetype) const { return name()!=mimetype; }
  bool operator <(const MimeType &mimetype) const
    { return ( name()<mimetype.name() ); }

  static const BasicMimeTypeList &all(bool withoutAlias);
  static const BasicMimeTypeList &all(FileType type);
  static const MimeType null;

private:
  QString     _aliasName, _name;
  const Data *_data;

  struct AllData {
    const char *mimetype;
    Data data;
  };
  static const AllData DATA[];
  static const Data NullData;
  static const Data UnspecifiedData;

  static QMap<QString, const Data *> *_mimetypes;
  static BasicMimeTypeList *_all, *_allwa;
  static QValueVector<BasicMimeTypeList> *_allTyped;

  void init(const QString &name, bool noUnalias);
  KMimeType::Ptr ptr() const { return KMimeType::mimeType(name()); }
  static QString unalias(const QString &mimetype);
  static bool isKnown(const QString &mimetype);
  static const Data &data(const QString &mimetype);
  static void initMimetypes();
  static void initAll();
  static QValueList<MimeType> allMimeTypes();
};

} // namespace

#endif
