#include "misc.h"

#include <qstringlist.h>
#include <qapplication.h>

#include <klocale.h>
#include <ktempfile.h>
#include <ktempdir.h>
#include <kio/jobclasses.h>
#include <kio/scheduler.h>
#include <kio/job.h>

using namespace Konvert;

//-----------------------------------------------------------------------------
const IOTypeData Konvert::IOTYPE_DATA[Nb_IOTypes] = {
  { "Source",      IO_ReadOnly,  ReadingErr },
  { "Destination", IO_WriteOnly, WritingErr }
};

const ResultData Konvert::RESULT_DATA[Nb_Results] = {
  { SuccessType,       I18N_NOOP("Success")              },
  { ErrorType,         I18N_NOOP("No filter available.") },
  { ErrorType,         I18N_NOOP("Error reading file.")  },
  { ErrorType,         I18N_NOOP("Error writing file.")  },
  { DetailedErrorType, I18N_NOOP("Error in filter: %1")  },
  { AbortedType,       I18N_NOOP("Aborted.")             },
  { AbortedType,       I18N_NOOP("Cancelled")            }
};

const ActionData Konvert::ACTION_DATA[Nb_Actions] = {
  { "convert",    I18N_NOOP("Convert"),    true  },
  { "extract",    I18N_NOOP("Extract"),    false },
  { "create",     I18N_NOOP("Create"),     true  },
  { "check",      I18N_NOOP("Checking"),   false }
};

//-----------------------------------------------------------------------------
TempFile::TempFile(Mode mode, DeleteMode dmode)
  : TempGeneric(dmode)
{
  KTempFile *tmp = new KTempFile();
  setName(tmp->name());
  tmp->unlink();
  delete tmp;
  if ( mode==Create ) open(IO_WriteOnly);
  qDebug("TempFile: %s autodelete=%i", name().latin1(), dmode==AutoDelete);
}

TempFile::~TempFile()
{
  qDebug("TempFile::delete %s exists=%i", name().latin1(), exists());
  if ( deleteMode()==AutoDelete ) unlink();
}

TempDir::TempDir(Mode mode, DeleteMode dmode)
  : TempGeneric(dmode)
{
  KTempDir *tmp = new KTempDir();
  setPath(tmp->name());
  if ( mode==DoNotCreate ) tmp->unlink();
  delete tmp;
  qDebug("TempDir: %s autodelete=%i", name().latin1(), dmode==AutoDelete);
}

bool TempDir::unlink()
{
  QString rmstr("/bin/rm -rf ");
  rmstr += KProcess::quote(name());
  return ( ::system(QFile::encodeName(rmstr))==0 );
}

TempDir::~TempDir()
{
  qDebug("TempDir::delete %s", name().latin1());
  if ( deleteMode()==AutoDelete ) unlink(); 
}

TempInode::TempInode(URLType type, Mode mode, DeleteMode dmode)
  : TempGeneric(dmode)
{
  if ( type==FileURL ) _tmp = new TempFile(mode, dmode);
  else _tmp = new TempDir(mode, dmode);
}

QFile *TempInode::file() const
{
  if ( _tmp->type()==DirectoryURL ) return 0;
  return static_cast<QFile *>(static_cast<const TempFile *>(_tmp));
}

QDir *TempInode::dir() const
{
  if ( _tmp->type()==FileURL ) return 0;
  return static_cast<QDir *>(static_cast<const TempDir *>(_tmp));
}

//-----------------------------------------------------------------------------
Process::Process(const QString &command, QObject *parent)
  : QObject(parent), _process(new KProcess(this)), _command(command)
{
  connect(_process, SIGNAL(receivedStderr(KProcess *, char *, int)),
          SLOT(readStderr(KProcess *, char *, int)));
  connect(_process, SIGNAL(receivedStdout(KProcess *, char *, int)),
          SLOT(readStdout(KProcess *, char *, int)));
}

void Process::readStderr(KProcess *, char *a, int buflen)
{
  addData(a, buflen, _err);
}

void Process::readStdout(KProcess *, char *a, int buflen)
{
  addData(a, buflen, _out);
}

void Process::addData(const char *a, uint buflen, QString &s)
{
  if ( buflen==0 ) return;
  QByteArray ba;
  ba.setRawData(a, buflen);
  s += QString(ba);
  ba.resetRawData(a, buflen);
}

bool Process::run(const QStringList &args,
                  KProcess::RunMode mode, KProcess::Communication com)
{
  _process->clearArguments();
  connect(_process, SIGNAL(processExited(KProcess *)), SLOT(processExitedSlot()));
  *_process << _command << args;
  QString s = _command;
  QStringList::const_iterator it = args.begin();
  for (; it!=args.end(); ++it) {
    s += " ";
    s += (*it);
  }
  qDebug("run command \"%s\"", s.latin1());
  return _process->start(mode, com);
}

void Process::abort()
{
  _process->disconnect(this);
  _process->kill();
}

void Process::processExitedSlot()
{
  qDebug("exited");
  _process->disconnect(this);
  emit processExited();
}

int Process::run(const QString &command, const QStringList &args,
                 const QString &workingDir)
{
  Process p(command);
  p.setWorkingDirectory(workingDir);
  if ( !p.run(args, KProcess::Block) ) return -1;
  if ( !p.normalExit() ) return -1;
  return p.exitStatus();
}

QStringList Process::args(const ArgList &list)
{
  QStringList args;
    for (uint i=0; list[i]; i++)
      args.append(list[i]);
  return args;
}

QStringList Process::args(const ArgData &data, const QString &source,
                          const QString &dest)
{
  QStringList list = args(data.options);
  switch (data.type) {
  case NoDest:
    list.append(source);
    break;
  case SourceDest:
    list.append(source);
    list.append(dest);
    break;
  case DestSource:
    list.append(dest);
    list.append(source);
    break;
  }
  return list;
}

KProcess::Communication Process::toCom(OutputType type)
{
  switch (type) {
  case NoOutput: return KProcess::NoCommunication;
  case Stdout: return KProcess::Stdout;
  case Stderr: return KProcess::Stderr;
  }
  Q_ASSERT(false);
  return KProcess::NoCommunication;
}

QString Process::output(OutputType type) const
{
  switch (type) {
  case NoOutput: return QString::null;
  case Stdout: return out();
  case Stderr: return err();
  }
  Q_ASSERT(false);
  return QString::null;
}

//----------------------------------------------------------------------------
Waiter::Waiter(QObject *object, const char *endSignal0, const char *endSignal1)
  : QObject(object)
{
  init();
  connect(object, endSignal0, SLOT(endSlot0()));
  if (endSignal1) connect(object, endSignal1, SLOT(endSlot1()));
}

int Waiter::wait()
{
  while ( !_done[0] && !_done[1] ) qApp->processEvents();
  int index = (_done[0] ? 0 : 1);
  init();
  return index;
}

void Waiter::init()
{
  _done[0] = false;
  _done[1] = false;
}

//----------------------------------------------------------------------------
DirLister::DirLister(QObject *parent, const char *name)
  : QObject(parent, name), _job(0), _type(Nb_FileTypes)
{}

void DirLister::setNameFilter(const QString &filter)
{
  _filter.clear();
  QStringList list = QStringList::split(' ', filter);
  QStringList::const_iterator it = list.begin();
  for (; it!=list.end(); ++it) {
    QRegExp r(*it, false, true);
    _filter.append(r);
  }
}

void DirLister::start(const KURL &dir, bool recurse, QWidget *widget)
{
  _dir = dir;
  _urls.clear();
  KIO::Scheduler::checkSlaveOnHold(true);
  if (recurse) _job = KIO::listRecursive(dir);
  else _job = KIO::listDir(dir);
  connect(_job, SIGNAL(entries(KIO::Job *, const KIO::UDSEntryList &)),
          SLOT(entries(KIO::Job *, const KIO::UDSEntryList &)));
  connect(_job, SIGNAL(result(KIO::Job *)), SIGNAL(done()));
  if (widget) _job->setWindow(widget);
}

void DirLister::abort()
{
  if (_job) _job->kill(false);
}

bool DirLister::hasError() const
{
  if (_job) return _job->error();
  return false;
}

bool DirLister::acceptEntry(const KFileItem &item) const
{
  if ( item.name()=="." || item.name()==".." ) return false;
  if ( (_exclusions & Directories) && item.isDir() ) return false;
  if ( (_exclusions & HiddenFiles) && item.name()[0]=="." )
    return false;
  if ( _filter.count()!=0 ) {
    QValueList<QRegExp>::const_iterator it = _filter.begin();
    for (; it!=_filter.end(); ++it)
      if ( (*it).exactMatch(item.name()) ) break;
    if ( it==_filter.end() ) return false;
  }
  return ( _type==Nb_FileTypes
           || MimeType::all(_type).contains(item.mimetype()) );
}

void DirLister::entries(KIO::Job *, const KIO::UDSEntryList &list)
{
  qDebug("entries slot");
  KIO::UDSEntryList::const_iterator itl = list.begin();
  for (; itl!=list.end(); ++itl) {
    KFileItem *item = new KFileItem(*itl, _dir, true, true);
    if ( acceptEntry(*item) ) _urls.append(item->url());
    delete item;
  }
}

#include "misc.moc"
