#ifndef MISC_H
#define MISC_H

#include <qfile.h>
#include <qdir.h>
#include <qregexp.h>

#include <kprocess.h>
#include <kfileitem.h>

#include "mimetype.h"

namespace KIO {
  class Job;
}

namespace Konvert
{
enum Result { Success = 0, NoFilterErr, ReadingErr, WritingErr,
              FilterErr, Aborted, Cancelled, Nb_Results };

enum IOType { Source = 0, Destination, Nb_IOTypes };
struct IOTypeData {
  const char *name;
  int openMode;
  Result openError;
};
extern const IOTypeData IOTYPE_DATA[Nb_IOTypes];

enum URLType { FileURL = 0, DirectoryURL, Nb_URLTypes };

enum Action { Convert = 0, Extract, Create, Check, Nb_Actions }; // !!!! DO NO REORDER
struct ActionData {
  const char *name, *label;
  bool needMimetype;
};
extern const ActionData ACTION_DATA[Nb_Actions];

enum ResultType { SuccessType, ErrorType, DetailedErrorType, AbortedType };
struct ResultData {
  ResultType type;
  const char *msg;
};
extern const ResultData RESULT_DATA[Nb_Results];

class ResData {
public:
  Result  result;
  QString detailedError;
};

//-----------------------------------------------------------------------------
class TempGeneric
{
public:
  enum Mode { Create, DoNotCreate };
  enum DeleteMode { AutoDelete, ManualDelete };
  TempGeneric(DeleteMode dmode) : _dmode(dmode)  {}
  virtual ~TempGeneric() {}

  virtual bool unlink() = 0;
  virtual QString name() const = 0;
  virtual URLType type() const = 0;
  DeleteMode deleteMode() const { return _dmode; }
  
private:
  DeleteMode _dmode;
};

class TempFile : public TempGeneric, public QFile
{
public:
  TempFile(Mode mode, DeleteMode dmode);
  virtual ~TempFile();

  virtual bool unlink() { return QFile::remove(); }
  virtual QString name() const { return QFile::name(); }
  virtual URLType type() const { return FileURL; }
};

class TempDir : public TempGeneric, public QDir
{
public:
  TempDir(Mode mode, DeleteMode dmode);
  virtual ~TempDir();

  virtual bool unlink();
  virtual QString name() const { return QDir::path(); }
  virtual URLType type() const { return DirectoryURL; }
};

// either a temporary file or directory
class TempInode : public TempGeneric
{
public:
  TempInode(URLType type, Mode mode, DeleteMode dmode);
  virtual ~TempInode() { delete _tmp; }

  virtual bool unlink() { return _tmp->unlink(); }
  virtual QString name() const { return _tmp->name(); }
  virtual URLType type() const { return _tmp->type(); }
  QFile *file() const;
  QDir *dir() const;

private:
  TempGeneric *_tmp;
};

//------------------------------------------------------------------------------
enum ArgType { NoDest, SourceDest, DestSource };
enum OutputType { NoOutput, Stdout, Stderr };

class Process : public QObject
{
  Q_OBJECT
public:
  Process(const QString &command, QObject *parent = 0);

  void setWorkingDirectory(const QString &dir) {
    _process->setWorkingDirectory(dir);
  }
  virtual bool run(const QStringList &args, KProcess::RunMode mode,
                   KProcess::Communication com = KProcess::NoCommunication);
  QString err() const { return _err; }
  QString out() const { return _out; }
  void abort();
  bool normalExit() const { return _process->normalExit(); }
  int exitStatus() const { return _process->exitStatus(); }
  QString command() const { return _command; }

  // return exist status (-1 if error or signalled)
  static int run(const QString &command, const QStringList &args,
                  const QString &workingDirectory = QString::null);
                  
  typedef const char *ArgList[10];
  struct ArgData {
    ArgType type;
    ArgList options;
  };
  static QStringList args(const ArgList &list);
  static QStringList args(const ArgData &data, const QString &source,
                          const QString &dest);
                          
  struct Data {
    const char *command;
    OutputType type;
    ArgList initArgs;
  };
  static KProcess::Communication toCom(OutputType type);
  QString output(OutputType type) const;

signals:
  void processExited();

protected slots:
  void readStderr(KProcess *, char *, int bufflen);
  void readStdout(KProcess *, char *, int bufflen);
  virtual void processExitedSlot();

protected:
  KProcess *_process;
  QString   _err, _out, _command;

  static void addData(const char *a, uint buflen, QString &s);
};

//----------------------------------------------------------------------------
class Waiter : public QObject
{
  Q_OBJECT
public:
  Waiter(QObject *object, const char *endSignal0, const char *endSignal1 = 0);
  int wait(); // return end signal index (0 or 1)
  
private slots:
  void endSlot0() { _done[0] = true; }
  void endSlot1() { _done[1] = true; }
  
private:
  bool _done[2];
  
  void init();
};

//----------------------------------------------------------------------------
class DirLister : public QObject
{
  Q_OBJECT
public:
  DirLister(QObject *parent = 0, const char *name = 0);

  enum Exclusion { HiddenFiles = 1, Directories = 2 };
  void setExclusions(int exclusions) { _exclusions = exclusions; }
  void setNameFilter(const QString &filter);
  void setMimeFilter(FileType type) { _type = type; }
  void start(const KURL &dir, bool recurse, QWidget *widget = 0);
  void abort();
  
  bool hasError() const;
  KURL::List urls() const { return _urls; }
  
signals:
  void done();
  
private slots:
  void entries(KIO::Job *, const KIO::UDSEntryList &);
    
private:
  KURL       _dir;
  KIO::Job  *_job;
  KURL::List _urls;
  int        _exclusions;
  QValueList<QRegExp> _filter;
  FileType _type;
  
  bool acceptEntry(const KFileItem &item) const;
};

} // namespace

#endif
