#include "publish_filter.h"

#include <qregexp.h>

#include "image_filter.h"

using namespace Konvert;

//----------------------------------------------------------------------------
void Konvert::addPublishingFilters(QPtrList<Filter> &list)
{
  for (uint i=0; i<PublishingFilter::Nb_Types; i++)
    list.append(new PublishingFilter(i));
  list.append(new GSPublishingFilter);
  list.append(new GenericConvertFilter(Publishing));
}

//----------------------------------------------------------------------------
const MultiTypeProcessFilter::CodeData
GSPublishingFilter::SOURCE_CODE_DATA[] = {
  {"PostScript", "application/postscript"},
  {"PDF",        "application/pdf"       },
  {0,  0}
};

const MultiTypeProcessFilter::CodeData
GSPublishingFilter::DEST_CODE_DATA[] = {
  {"pswrite",  "application/postscript"},
  {"pdfwrite", "application/pdf"       },
  {"png256",   "image/png"             },
  {"epswrite", "image/x-eps"           },
  {0,  0}
};

const MultiTypeProcessFilter::Data GSPublishingFilter::DATA = {
  {"gs", Stdout, "-help", 0}, false, {SOURCE_CODE_DATA, DEST_CODE_DATA}
};

GSPublishingFilter::GSPublishingFilter()
  : MultiTypeProcessFilter(DATA, Convert, Publishing)
{}

QStringList GSPublishingFilter::extractCodes(IOType type,
                                             const QStringList &lines) const
{
  QStringList list;
  QString start = (type==Source ? "Input formats" : "Available devices");
  bool found = false;
  QRegExp regExp("^" + start + ":\\s*([\\w\\s]*)");
  QStringList::const_iterator it = lines.begin();
  for (; it!=lines.end(); ++it) {
    int res = regExp.search(*it);
    if ( res==-1 ) {
       if (found) break;
       continue;
    }
    list += QStringList::split(' ', regExp.cap(1));
    if ( !found) {
      found = true;
      regExp.setPattern("^\\s+([\\w\\s]+)");
    }
  }
  return list;
}  

QStringList GSPublishingFilter::args(const QString &source, const MimeType &,
                                     const QString &dest, const MimeType &destType) const
{
  QStringList args;
  args.append("-q"); // quiet
  args.append("-dNOPAUSE"); // no pause between pages
  args.append("-dBATCH"); // exit after last file
  args.append("-dSAFER"); // safe against malicious script
  args.append("-sDEVICE=" + findCode(Destination, destType));
  args.append("-sOutputFile=" + dest);
  args.append("-c");
  if ( destType.name()=="application/pdf" ) args.append(".setpdfwrite"); // ??
  else {
    args.append("save");
    // helps keep font from being flushed between pages
    if ( destType.name()=="application/postscript" ) args.append("pop");
  }
  args.append("-f"); args.append(source);
  return args;
}

//----------------------------------------------------------------------------
// Note: pdf2ps, ps2pdf, ps2epsi are all scripts using 'gs'
//       ps2ascii uses 'gs' but in a strange way: so use it directly
//       dvipdf is a script using 'dvips' and 'gs'
//       dvi2fax is a script using 'dvips' and 'gs'
//       epstopdf is an Image filter (since eps is considered Image for the time being)
const PublishingFilter::ExtData PublishingFilter::DATA[Nb_Types] = {
  {{"pdftops", NoOutput, "-v", 0}, {SourceDest, 0}, {{"application/pdf", 0}, {"application/postscript", 0}}}, // xpdf
  {{"epstopdf", NoOutput, "--help", 0}, {DestSource, "-o=", 0}, {{"image/x-eps", 0}, {"application/pdf", 0}}},
  {{"dvips", NoOutput, "-v", 0}, {DestSource, "-o", 0}, {{"application/x-dvi", 0}, {"application/postscript", 0}}},
  {{"pdftotext", NoOutput, "-v", 0}, {SourceDest, 0}, {{"application/pdf", 0}, {"text/plain", 0}}}, // xpdf
  {{"ps2ascii", NoOutput, "-v", 0}, {SourceDest, 0}, {{"application/postscript", "application.pdf", 0}, {"text/plain", 0}}} // gs
};

PublishingFilter::PublishingFilter(uint i)
  : ProcessFilter(DATA[i].data, MimeType(DATA[i].mimetypes[Source][0]).data().type, Convert),
    _data(DATA[i])
{}

BasicMimeTypeList PublishingFilter::findSupportedMimetypes(IOType type,
                                         const QStringList &) const
{
  BasicMimeTypeList list;
  for (uint i=0; _data.mimetypes[type][i]; i++)
    list.insert(MimeType(_data.mimetypes[type][i]));
  return list;
}

QStringList PublishingFilter::args(const QString &source, const MimeType &,
                                   const QString &dest, const MimeType &) const
{
  return Process::args(_data.args, source, dest);
}

#include "publish_filter.moc"
