#include "transform.h"

#include <klocale.h>
#include <ktempfile.h>
#include <ktempdir.h>

#include "filter.h"

using namespace Konvert;

//-----------------------------------------------------------------------------
const TransActionData Konvert::TRANS_ACTION_DATA[Nb_TransActions] = {
  { "extract", Source      },
  { "create",  Destination },
  { "check",   Source      }
};

//-----------------------------------------------------------------------------
const ProcessTransform::SubData ProcessTransform::NOT_SUPPORTED =
  {false, {NoDest, 0}};

ProcessTransform::ProcessTransform(const Data &data)
  : _data(data), _initialized(false)
{}

void ProcessTransform::initSupported() const
{
  if (_initialized) return;
  _initialized = true;
  
  // program present ?
  if ( _data.data.command==0 ) return;
  QStringList args;
  args = Process::args(_data.data.initArgs);
  Process process(_data.data.command);
  if ( !process.run(args, KProcess::Block, Process::toCom(_data.data.type)) ) return;
  if ( !process.normalExit() || process.exitStatus()<0 ) return;

  // supported mimetypes
  _supported.insert(findSupportedMimetypes(process.output(_data.data.type)));
}

QStringList ProcessTransform::findSupportedMimetypes(const QString &) const
{
  Q_ASSERT(_data.mimetype);
  return QStringList(_data.mimetype);
}

bool ProcessTransform::supports(TransAction action,
                                const MimeType &mimetype) const
{
  initSupported();
  if ( !_data.sub[action].supported ) return false;
  return _supported.contains(mimetype);
}

QStringList ProcessTransform::args(TransAction action, const QString &source,
                                  const QString &dest) const
{
  return Process::args(_data.sub[action].args, source, dest);
}

bool ProcessTransform::doAction(TransAction action, const MimeType &,
                          const ExchangeData &source, const ExchangeData &dest,
                          Filter *filter) const
{
  QStringList a = args(action, source.dataFilename, dest.dataFilename);
  int res = Process::run(_data.data.command, a, dest.dataFilename);
  if ( res!=0 ) {
    if (filter) filter->done(FilterErr, "Error proccessing file.");
    return false;
  }
  return true;
}

//-----------------------------------------------------------------------------
const TransformFilter::Data TransformFilter::DATA[Nb_Actions] = {
  { TransExtract,    TransCreate     },
  { TransExtract,    Nb_TransActions },
  { Nb_TransActions, TransCreate     },
  { TransCheck,      Nb_TransActions }
};

TransformFilter::TransformFilter(KStaticDeleter<TransformList> &sd,
                                 TransformListPtr &list,
                                 URLType baseType, FileType type, Action action)
  : Filter("tranform", type, action), _baseType(baseType), _sd(sd), _list(list)
{}

BasicMimeTypeList TransformFilter::getSupportedMimetypes(IOType ioType) const
{
  qDebug("TransformFilter::getSupportedMimetypes %s", IOTYPE_DATA[ioType].name);
  BasicMimeTypeList list;
  TransAction tAction = DATA[action()].action[ioType];
  bool allMimetypes = false;
  switch (action()) {
  case Create: allMimetypes = ( ioType==Source ); break;
  case Convert: allMimetypes = ( ioType==Destination ); break;
  case Check:
  case Extract: break;
  case Nb_Actions: Q_ASSERT(false); break;
  };
  const BasicMimeTypeList &all =
    (allMimetypes ? MimeType::all(true) : MimeType::all(type()));
  BasicMimeTypeList::const_iterator it = all.begin();
  for (; it!=all.end(); ++it) {
    if ( isSupported(action(), tAction, ioType, it.key()) ) {
//      qDebug("TransformFilter::getSupportedMimetypes %i %s %s", ioType,
//             ACTION_DATA[action()].name, it.key().name().latin1());
      list.insert(it.key());
    }
  }
  return list;
}

bool TransformFilter::doAction(TransAction action, const MimeType &mimetype,
                          const ExchangeData &source, const ExchangeData &dest)
{
  return find(action, mimetype)
    ->doAction(action, mimetype, source, dest, this);
}

void TransformFilter::startInternal(const QString &source,
     const MimeType &sourceType, const QString &dest, const MimeType &destType)
{
  ExchangeData src, tmp, dst;
  src.dataFilename = source;
  dst.dataFilename = dest;
  switch (action()) {
  case Convert: {
    TempInode::Mode mode = (_baseType==FileURL ? TempInode::DoNotCreate : TempInode::Create);
    TempInode inode(_baseType, mode, TempInode::AutoDelete);
    tmp.dataFilename = inode.name();
    if ( !doAction(TransExtract, sourceType, src, tmp) ) return;
    if ( !doAction(TransCreate, destType, tmp, dst) ) return;
    break;
  }
  case Extract:
    if ( !doAction(TransExtract, sourceType, src, dst) ) return;
    break;
  case Create:
    if ( !doAction(TransCreate, destType, src, dst) ) return;
    break;
  case Check:
    if ( !doAction(TransCheck, sourceType, src, tmp) ) return;
    break;
  case Nb_Actions: Q_ASSERT(false); break;
  }
  done(Success);
}

URLType TransformFilter::urlType(IOType type, const MimeType &sourceType) const
{
  if ( type==Destination && action()==Extract ) return _baseType;
  return Filter::urlType(type, sourceType);
}

GenericTransform *TransformFilter::find(TransAction action,
                                        const MimeType &mimetype) const
{
  initList();
  QPtrListIterator<GenericTransform> it(*_list);
  for (; it.current(); ++it)
    if ( it.current()->supports(action, mimetype) ) return it.current();
  return 0;
}

bool TransformFilter::isSupported(TransAction action,
                                  const MimeType &mimetype) const
{
  return find(action, mimetype);
}

bool TransformFilter::isSupported(Action, TransAction tAction,
                                  IOType, const MimeType &mimetype) const
{
  return isSupported(tAction, mimetype);
}

void TransformFilter::initList() const
{
  if (_list) return;
  _sd.setObject(_list, new TransformList);
  _list->setAutoDelete(true);
  fillList(*_list);
}

#include "transform.moc"
